@@ -4,7 +4,7 @@ namespace {
 
 class Deinterleave : public Halide::Generator<Deinterleave> {
 public:
-    Input<Buffer<uint8_t>> uvInterleaved{"uvInterleaved", 2};
+    Input<Buffer<uint8_t, 2>> uvInterleaved{"uvInterleaved"};
     // There is no way to declare a Buffer<Tuple>, so we must use Output<Func> instead
     Output<Func> result{"result", {UInt(8), UInt(8)}, 2};
 
