@@ -36,7 +36,8 @@ struct ApplySplitResult {
                 LetStmt,
                 PredicateCalls,
                 PredicateProvides,
-                Predicate };
+                Predicate,
+                BlendProvides };
     Type type;
 
     ApplySplitResult(const std::string &n, Expr val, Type t)
@@ -67,6 +68,9 @@ struct ApplySplitResult {
     bool is_predicate_provides() const {
         return (type == PredicateProvides);
     }
+    bool is_blend_provides() const {
+        return (type == BlendProvides);
+    }
 };
 
 /** Given a Split schedule on a definition (init or update), return a list of
