@@ -22,10 +22,10 @@ using std::sort;
 using std::string;
 using std::vector;
 
-static ostringstream nil;
-
 namespace {
 
+ostringstream nil;
+
 class CodeGen_D3D12Compute_Dev : public CodeGen_GPU_Dev {
 public:
     CodeGen_D3D12Compute_Dev(const Target &target);
@@ -947,7 +947,6 @@ string CodeGen_D3D12Compute_Dev::CodeGen_D3D12Compute_C::print_cast(Type target_
 void CodeGen_D3D12Compute_Dev::CodeGen_D3D12Compute_C::visit(const Cast *op) {
     Type target_type = op->type;
     Type source_type = op->value.type();
-    string value_expr = print_expr(op->value);
 
     string cast_expr = print_cast(target_type, source_type, print_expr(op->value));
 
