@@ -175,6 +175,17 @@ void CodeGen_WebAssembly::visit(const Cast *op) {
                 }
             }
         }
+
+        // Narrowing float -> int casts should go via an integer type of the
+        // matching width (see https://github.com/halide/Halide/issues/7972)
+        if (op->value.type().is_float() &&
+            (op->type.is_int() || op->type.is_uint()) &&
+            op->type.bits() < op->value.type().bits()) {
+            Expr equiv = Cast::make(op->type.with_bits(op->value.type().bits()), op->value);
+            equiv = Cast::make(op->type, equiv);
+            codegen(equiv);
+            return;
+        }
     }
 
     CodeGen_Posix::visit(op);
@@ -319,42 +330,36 @@ string CodeGen_WebAssembly::mcpu_tune() const {
 }
 
 string CodeGen_WebAssembly::mattrs() const {
-    std::ostringstream s;
-    string sep;
-
-    if (target.has_feature(Target::WasmSignExt)) {
-        s << sep << "+sign-ext";
-        sep = ",";
-    }
-
-    if (target.has_feature(Target::WasmSimd128)) {
-        s << sep << "+simd128";
-        sep = ",";
-    }
-
-    if (target.has_feature(Target::WasmSatFloatToInt)) {
-        s << sep << "+nontrapping-fptoint";
-        sep = ",";
-    }
-
-    if (target.has_feature(Target::WasmThreads)) {
-        // "WasmThreads" doesn't directly affect LLVM codegen,
-        // but it does end up requiring atomics, so be sure to enable them.
-        s << sep << ",+atomics";
-        sep = ",";
-    }
-
-    // Recent Emscripten builds assume that specifying `-pthread` implies bulk-memory too,
-    // so quietly enable it if either of these are specified.
-    if (target.has_feature(Target::WasmBulkMemory) || target.has_feature(Target::WasmThreads)) {
-        s << sep << "+bulk-memory";
-        sep = ",";
-    }
-
     user_assert(target.os == Target::WebAssemblyRuntime)
         << "wasmrt is the only supported 'os' for WebAssembly at this time.";
 
-    return s.str();
+    std::vector<std::string_view> attrs;
+
+    if (!target.has_feature(Target::WasmMvpOnly)) {
+        attrs.emplace_back("+sign-ext");
+        attrs.emplace_back("+nontrapping-fptoint");
+    }
+    if (target.has_feature(Target::WasmSimd128)) {
+        attrs.emplace_back("+simd128");
+    }
+    if (target.has_feature(Target::WasmThreads)) {
+        // "WasmThreads" doesn't directly affect LLVM codegen,
+        // but it does end up requiring atomics, so be sure to enable them.
+        attrs.emplace_back("+atomics");
+    }
+    // PIC implies +mutable-globals because the PIC ABI used by the linker
+    // depends on importing and exporting mutable globals. Also -pthread implies
+    // mutable-globals too, so quitely enable it if either of these are specified.
+    if (use_pic() || target.has_feature(Target::WasmThreads)) {
+        attrs.emplace_back("+mutable-globals");
+    }
+    // Recent Emscripten builds assume that specifying `-pthread` implies bulk-memory too,
+    // so quietly enable it if either of these are specified.
+    if (target.has_feature(Target::WasmBulkMemory) || target.has_feature(Target::WasmThreads)) {
+        attrs.emplace_back("+bulk-memory");
+    }
+
+    return join_strings(attrs, ",");
 }
 
 bool CodeGen_WebAssembly::use_soft_float_abi() const {
@@ -362,7 +367,18 @@ bool CodeGen_WebAssembly::use_soft_float_abi() const {
 }
 
 bool CodeGen_WebAssembly::use_pic() const {
+#if LLVM_VERSION >= 180
+    // Issues with WASM PIC and dynamic linking only got fixed in LLVM v18.x (June 26th 2023)
+    // See https://reviews.llvm.org/D153293
+
+    // Always emitting PIC "does add a little bloat to the object files, due to the extra
+    // indirection, but when linked into a static binary 100% of this can be removed by
+    // wasm-opt in release builds."
+    // See https://github.com/halide/Halide/issues/7796
+    return true;
+#else
     return false;
+#endif
 }
 
 int CodeGen_WebAssembly::native_vector_bits() const {
