@@ -23,6 +23,8 @@ enum class DeviceAPI {
     Hexagon,
     HexagonDma,
     D3D12Compute,
+    Vulkan,
+    WebGPU,
 };
 
 /** An array containing all the device apis. Useful for iterating
@@ -36,7 +38,9 @@ const DeviceAPI all_device_apis[] = {DeviceAPI::None,
                                      DeviceAPI::Metal,
                                      DeviceAPI::Hexagon,
                                      DeviceAPI::HexagonDma,
-                                     DeviceAPI::D3D12Compute};
+                                     DeviceAPI::D3D12Compute,
+                                     DeviceAPI::Vulkan,
+                                     DeviceAPI::WebGPU};
 
 }  // namespace Halide
 
