@@ -8,6 +8,7 @@
 
 #include <map>
 #include <string>
+#include <vector>
 
 #include "Expr.h"
 
@@ -22,18 +23,18 @@ class Function;
  *  _does not_ include the Function f, unless it is called recursively by
  *  itself.
  */
-std::map<std::string, Function> find_direct_calls(Function f);
+std::map<std::string, Function> find_direct_calls(const Function &f);
 
 /** Construct a map from name to Function definition object for all Halide
  *  functions called directly in the definition of the Function f, or
  *  indirectly in those functions' definitions, recursively. This map always
  *  _includes_ the Function f.
  */
-std::map<std::string, Function> find_transitive_calls(Function f);
+std::map<std::string, Function> find_transitive_calls(const Function &f);
 
-/** Find all Functions transitively referenced by f in any way and add
- * them to the given map. */
-void populate_environment(Function f, std::map<std::string, Function> &env);
+/** Find all Functions transitively referenced by any Function in `funcs` and return
+ * a map of them. */
+std::map<std::string, Function> build_environment(const std::vector<Function> &funcs);
 
 }  // namespace Internal
 }  // namespace Halide
