@@ -299,11 +299,12 @@ public:
     }
 };
 
-// A counter to use in tagging random variables
-std::atomic<int> rand_counter{0};
-
 }  // namespace
 
+// A counter to use in tagging random variables.
+// Note that this will be reset by Internal::reset_random_counters().
+std::atomic<int> random_variable_counter = 0;
+
 Function::Function(const FunctionPtr &ptr)
     : contents(ptr) {
     contents.strengthen();
@@ -331,6 +332,50 @@ Function::Function(const std::vector<Type> &required_types, int required_dims, c
     contents->required_dims = required_dims;
 }
 
+void Function::update_with_deserialization(const std::string &name,
+                                           const std::string &origin_name,
+                                           const std::vector<Halide::Type> &output_types,
+                                           const std::vector<Halide::Type> &required_types,
+                                           int required_dims,
+                                           const std::vector<std::string> &args,
+                                           const FuncSchedule &func_schedule,
+                                           const Definition &init_def,
+                                           const std::vector<Definition> &updates,
+                                           const std::string &debug_file,
+                                           const std::vector<Parameter> &output_buffers,
+                                           const std::vector<ExternFuncArgument> &extern_arguments,
+                                           const std::string &extern_function_name,
+                                           NameMangling name_mangling,
+                                           DeviceAPI device_api,
+                                           const Expr &extern_proxy_expr,
+                                           bool trace_loads,
+                                           bool trace_stores,
+                                           bool trace_realizations,
+                                           const std::vector<std::string> &trace_tags,
+                                           bool frozen) {
+    contents->name = name;
+    contents->origin_name = origin_name;
+    contents->output_types = output_types;
+    contents->required_types = required_types;
+    contents->required_dims = required_dims;
+    contents->args = args;
+    contents->func_schedule = func_schedule;
+    contents->init_def = init_def;
+    contents->updates = updates;
+    contents->debug_file = debug_file;
+    contents->output_buffers = output_buffers;
+    contents->extern_arguments = extern_arguments;
+    contents->extern_function_name = extern_function_name;
+    contents->extern_mangling = name_mangling;
+    contents->extern_function_device_api = device_api;
+    contents->extern_proxy_expr = extern_proxy_expr;
+    contents->trace_loads = trace_loads;
+    contents->trace_stores = trace_stores;
+    contents->trace_realizations = trace_realizations;
+    contents->trace_tags = trace_tags;
+    contents->frozen = frozen;
+}
+
 namespace {
 
 template<typename T>
@@ -542,7 +587,7 @@ void Function::define(const vector<string> &args, vector<Expr> values) {
     }
 
     // Tag calls to random() with the free vars
-    int tag = rand_counter++;
+    int tag = random_variable_counter++;
     vector<VarOrRVar> free_vars;
     free_vars.reserve(args.size());
     for (const auto &arg : args) {
@@ -752,7 +797,7 @@ void Function::define_update(const vector<Expr> &_args, vector<Expr> values) {
             free_vars.emplace_back(RVar(check.reduction_domain, i));
         }
     }
-    int tag = rand_counter++;
+    int tag = random_variable_counter++;
     for (auto &arg : args) {
         arg = lower_random(arg, free_vars, tag);
     }
@@ -1101,12 +1146,16 @@ void Function::lock_loop_levels() {
     auto &schedule = contents->func_schedule;
     schedule.compute_level().lock();
     schedule.store_level().lock();
+    schedule.hoist_storage_level().lock();
     // If store_level is inlined, use the compute_level instead.
     // (Note that we deliberately do *not* do the same if store_level
     // is undefined.)
     if (schedule.store_level().is_inlined()) {
         schedule.store_level() = schedule.compute_level();
     }
+    if (schedule.hoist_storage_level().is_inlined()) {
+        schedule.hoist_storage_level() = schedule.store_level();
+    }
     if (contents->init_def.defined()) {
         contents->init_def.schedule().fuse_level().level.lock();
     }
