@@ -10,6 +10,7 @@
 #include "Generator.h"
 #include "IRPrinter.h"
 #include "Module.h"
+#include "Serialization.h"
 #include "Simplify.h"
 
 #ifdef HALIDE_ALLOW_GENERATOR_BUILD_METHOD
@@ -589,12 +590,15 @@ const std::map<std::string, Type> &get_halide_type_enum_map() {
         {"int8", Int(8)},
         {"int16", Int(16)},
         {"int32", Int(32)},
+        {"int64", Int(64)},
         {"uint8", UInt(8)},
         {"uint16", UInt(16)},
         {"uint32", UInt(32)},
+        {"uint64", UInt(64)},
         {"float16", Float(16)},
         {"float32", Float(32)},
-        {"float64", Float(64)}};
+        {"float64", Float(64)},
+        {"bfloat16", BFloat(16)}};
     return halide_type_enum_map;
 }
 
@@ -651,7 +655,8 @@ gengen
  -e  A comma separated list of files to emit. Accepted values are:
      [assembly, bitcode, c_header, c_source, cpp_stub, featurization,
       llvm_assembly, object, python_extension, pytorch_wrapper, registration,
-      schedule, static_library, stmt, stmt_html, compiler_log].
+      schedule, static_library, stmt, stmt_html, conceptual_stmt,
+      conceptual_stmt_html, compiler_log, hlpipe, device_code].
      If omitted, default value is [c_header, static_library, registration].
 
  -p  A comma-separated list of shared libraries that will be loaded before the
@@ -661,15 +666,15 @@ gengen
      (Note that this does not change the default autoscheduler; use the -s flag
      to set that value.)"
 
- -r   The name of a standalone runtime to generate. Only honors EMIT_OPTIONS 'o'
+ -r  The name of a standalone runtime to generate. Only honors EMIT_OPTIONS 'o'
      and 'static_library'. When multiple targets are specified, it picks a
      runtime that is compatible with all of the targets, or fails if it cannot
      find one. Flags across all of the targets that do not affect runtime code
      generation, such as `no_asserts` and `no_runtime`, are ignored.
 
  -t  Timeout for the Generator to run, in seconds; mainly useful to ensure that
-     bugs and/or degenerate cases don't stall build systems. Defaults to 900
-     (=15 minutes). Specify 0 to allow ~infinite time.
+     bugs and/or degenerate cases don't stall build systems. Specify 0 to allow
+     infinite time. Defaults to infinite.
 
  -v  If nonzero, log the path to all generated files to stdout.
 )INLINE_CODE";
@@ -684,7 +689,7 @@ gengen
         {"-p", ""},
         {"-r", ""},
         {"-v", "0"},
-        {"-t", "900"},  // 15 minutes
+        {"-t", "0"},
     };
 
     ExecuteGeneratorArgs args;
@@ -772,6 +777,12 @@ gengen
         std::set<OutputFileType> output_types;
 
         std::string emit_flags_string = flags_info["-e"];
+
+        // If omitted or empty, assume .a and .h and registration.cpp
+        if (emit_flags_string.empty()) {
+            emit_flags_string = "c_header,registration,static_library";
+        }
+
         // If HL_EXTRA_OUTPUTS is defined, assume it's extra outputs we want to generate
         // (usually for temporary debugging purposes) and just tack it on to the -e contents.
         std::string extra_outputs = get_env_variable("HL_EXTRA_OUTPUTS");
@@ -783,49 +794,45 @@ gengen
         }
 
         const std::vector<std::string> emit_flags = split_string(emit_flags_string, ",");
+        internal_assert(!emit_flags.empty()) << "Empty emit flags.\n";
 
-        if (emit_flags.empty() || (emit_flags.size() == 1 && emit_flags[0].empty())) {
-            // If omitted or empty, assume .a and .h and registration.cpp
-            output_types.insert(OutputFileType::c_header);
-            output_types.insert(OutputFileType::registration);
-            output_types.insert(OutputFileType::static_library);
-        } else {
-            // Build a reverse lookup table. Allow some legacy aliases on the command line,
-            // to allow legacy build systems to work more easily.
-            std::map<std::string, OutputFileType> output_name_to_enum = {
-                {"cpp", OutputFileType::c_source},
-                {"h", OutputFileType::c_header},
-                {"html", OutputFileType::stmt_html},
-                {"o", OutputFileType::object},
-                {"py.c", OutputFileType::python_extension},
-            };
-            // extensions won't vary across multitarget output
-            const Target t = args.targets.empty() ? Target() : args.targets[0];
-            const std::map<OutputFileType, const OutputInfo> output_info = get_output_info(t);
-            for (const auto &it : output_info) {
-                output_name_to_enum[it.second.name] = it.first;
-            }
-
-            for (const std::string &opt : emit_flags) {
-                auto it = output_name_to_enum.find(opt);
-                if (it == output_name_to_enum.end()) {
-                    std::ostringstream o;
-                    o << "Unrecognized emit option: " << opt << " is not one of [";
-                    auto end = output_info.cend();
-                    auto last = std::prev(end);
-                    for (auto iter = output_info.cbegin(); iter != end; ++iter) {
-                        o << iter->second.name;
-                        if (iter != last) {
-                            o << " ";
-                        }
-                    }
-                    o << "], ignoring.\n";
-                    o << kUsage;
-                    user_error << o.str();
-                }
-                output_types.insert(it->second);
-            }
+        // Build a reverse lookup table. Allow some legacy aliases on the command line,
+        // to allow legacy build systems to work more easily.
+        std::map<std::string, OutputFileType> output_name_to_enum = {
+            {"cpp", OutputFileType::c_source},
+            {"h", OutputFileType::c_header},
+            {"hlpipe", OutputFileType::hlpipe},
+            {"html", OutputFileType::stmt_html},
+            {"o", OutputFileType::object},
+            {"py.c", OutputFileType::python_extension},
+        };
+        // extensions won't vary across multitarget output
+        const Target t = args.targets.empty() ? Target() : args.targets[0];
+        const std::map<OutputFileType, const OutputInfo> output_info = get_output_info(t);
+        for (const auto &it : output_info) {
+            output_name_to_enum[it.second.name] = it.first;
         }
+
+        for (const std::string &opt : emit_flags) {
+            auto it = output_name_to_enum.find(opt);
+            if (it == output_name_to_enum.end()) {
+                std::ostringstream o;
+                o << "Unrecognized emit option: " << opt << " is not one of [";
+                auto end = output_info.cend();
+                auto last = std::prev(end);
+                for (auto iter = output_info.cbegin(); iter != end; ++iter) {
+                    o << iter->second.name;
+                    if (iter != last) {
+                        o << " ";
+                    }
+                }
+                o << "], ignoring.\n";
+                o << kUsage;
+                user_error << o.str();
+            }
+            output_types.insert(it->second);
+        }
+
         return output_types;
     };
 
@@ -998,6 +1005,7 @@ void execute_generator(const ExecuteGeneratorArgs &args_in) {
 
     const bool cpp_stub_only = args.output_types.size() == 1 &&
                                args.output_types.count(OutputFileType::cpp_stub) == 1;
+
     if (!cpp_stub_only) {
         // It's ok to leave targets unspecified if we are generating *only* a cpp_stub
         internal_assert(!args.targets.empty());
@@ -1047,8 +1055,22 @@ void execute_generator(const ExecuteGeneratorArgs &args_in) {
     if (!args.generator_name.empty()) {
         const std::string base_path = args.output_dir + "/" + args.file_base_name;
         debug(1) << "Generator " << args.generator_name << " has base_path " << base_path << "\n";
+
+        // Factory function for creating a generator instance for a given function name and target
+        auto generator_factory = [&](const std::string &function_name, const Target &target) -> AbstractGeneratorPtr {
+            // Must re-create each time since each instance will have a different Target.
+            auto gen = args.create_generator(args.generator_name, GeneratorContext(target));
+            for (const auto &kv : args.generator_params) {
+                if (kv.first == "target") {
+                    continue;
+                }
+                gen->set_generatorparam_value(kv.first, kv.second);
+            }
+            return gen;
+        };
+
         if (args.output_types.count(OutputFileType::cpp_stub)) {
-            // When generating cpp_stub, we ignore all generator args passed in, and supply a fake Target.
+            // When generating cpp_stub we ignore all generator args passed in, and supply a fake Target.
             // (CompilerLogger is never enabled for cpp_stub, for now anyway.)
             const Target fake_target = Target();
             auto gen = args.create_generator(args.generator_name, GeneratorContext(fake_target));
@@ -1056,18 +1078,26 @@ void execute_generator(const ExecuteGeneratorArgs &args_in) {
             gen->emit_cpp_stub(output_files[OutputFileType::cpp_stub]);
         }
 
+#ifdef WITH_SERIALIZATION
+        if (args.output_types.count(OutputFileType::hlpipe)) {
+            // When serializing a halide pipeline, target is required (since the schedule may be target dependent).
+            // If multiple targets are specified, add the target name as a suffix to the filename.
+            const bool use_target_suffix = (args.targets.size() > 1);
+            for (size_t i = 0; i < args.targets.size(); ++i) {
+                const Target &target = args.targets[i];
+                const std::string hlpipe_path = use_target_suffix ? (base_path + "-" + target.to_string()) : base_path;
+                auto output_files = compute_output_files(target, hlpipe_path, args.output_types);
+                auto gen = generator_factory(args.function_name, target);
+                gen->emit_hlpipe(output_files[OutputFileType::hlpipe]);
+            }
+        }
+#endif
+
         // Don't bother with this if we're just emitting a cpp_stub.
         if (!cpp_stub_only) {
             auto output_files = compute_output_files(args.targets[0], base_path, args.output_types);
             auto module_factory = [&](const std::string &function_name, const Target &target) -> Module {
-                // Must re-create each time since each instance will have a different Target.
-                auto gen = args.create_generator(args.generator_name, GeneratorContext(target));
-                for (const auto &kv : args.generator_params) {
-                    if (kv.first == "target") {
-                        continue;
-                    }
-                    gen->set_generatorparam_value(kv.first, kv.second);
-                }
+                auto gen = generator_factory(function_name, target);
                 return args.build_mode == ExecuteGeneratorArgs::Gradient ?
                            gen->build_gradient_module(function_name) :
                            gen->build_module(function_name);
@@ -1218,10 +1248,6 @@ GeneratorBase::~GeneratorBase() {
 }
 
 GeneratorParamInfo::GeneratorParamInfo(GeneratorBase *generator, const size_t size) {
-    std::vector<void *> vf = ObjectInstanceRegistry::instances_in_range(
-        generator, size, ObjectInstanceRegistry::FilterParam);
-    user_assert(vf.empty()) << "ImageParam and Param<> are no longer allowed in Generators; use Input<> instead.";
-
     const auto add_synthetic_params = [this, generator](GIOBase *gio) {
         const std::string &n = gio->name();
         const std::string &gn = generator->generator_registered_name;
@@ -1239,45 +1265,68 @@ GeneratorParamInfo::GeneratorParamInfo(GeneratorBase *generator, const size_t si
         }
     };
 
-    std::vector<void *> vi = ObjectInstanceRegistry::instances_in_range(
-        generator, size, ObjectInstanceRegistry::GeneratorInput);
-    for (auto *v : vi) {
-        auto *input = static_cast<Internal::GeneratorInputBase *>(v);
-        internal_assert(input != nullptr);
-        user_assert(is_valid_name(input->name())) << "Invalid Input name: (" << input->name() << ")\n";
-        user_assert(!names.count(input->name())) << "Duplicate Input name: " << input->name();
-        names.insert(input->name());
-        internal_assert(input->generator == nullptr || input->generator == generator);
-        input->generator = generator;
-        filter_inputs.push_back(input);
-        add_synthetic_params(input);
-    }
+    const char *const io_order_warning = "Generators will always produce code that orders all Inputs before all Outputs; "
+                                         "this Generator declares the Inputs and Outputs in a different order, so the calling convention may not be as expected. "
+                                         "A future version of Halide will make this illegal, and require all Inputs to be declared before all Outputs. "
+                                         "(You can avoid this requirement by overriding Generator::allow_out_of_order_inputs_and_outputs().)";
 
-    std::vector<void *> vo = ObjectInstanceRegistry::instances_in_range(
-        generator, size, ObjectInstanceRegistry::GeneratorOutput);
-    for (auto *v : vo) {
-        auto *output = static_cast<Internal::GeneratorOutputBase *>(v);
-        internal_assert(output != nullptr);
-        user_assert(is_valid_name(output->name())) << "Invalid Output name: (" << output->name() << ")\n";
-        user_assert(!names.count(output->name())) << "Duplicate Output name: " << output->name();
-        names.insert(output->name());
-        internal_assert(output->generator == nullptr || output->generator == generator);
-        output->generator = generator;
-        filter_outputs.push_back(output);
-        add_synthetic_params(output);
-    }
-
-    std::vector<void *> vg = ObjectInstanceRegistry::instances_in_range(
-        generator, size, ObjectInstanceRegistry::GeneratorParam);
-    for (auto *v : vg) {
-        auto *param = static_cast<GeneratorParamBase *>(v);
-        internal_assert(param != nullptr);
-        user_assert(is_valid_name(param->name())) << "Invalid GeneratorParam name: " << param->name();
-        user_assert(!names.count(param->name())) << "Duplicate GeneratorParam name: " << param->name();
-        names.insert(param->name());
-        internal_assert(param->generator == nullptr || param->generator == generator);
-        param->generator = generator;
-        filter_generator_params.push_back(param);
+    bool outputs_seen = false;
+    auto vf = ObjectInstanceRegistry::instances_in_range(generator, size);
+    for (const auto &vi : vf) {
+        void *const instance = vi.first;
+        const ObjectInstanceRegistry::Kind kind = vi.second;
+        switch (kind) {
+        case ObjectInstanceRegistry::GeneratorParam: {
+            auto *param = static_cast<GeneratorParamBase *>(instance);
+            internal_assert(param != nullptr);
+            user_assert(is_valid_name(param->name())) << "Invalid GeneratorParam name: " << param->name();
+            user_assert(!names.count(param->name())) << "Duplicate GeneratorParam name: " << param->name();
+            names.insert(param->name());
+            internal_assert(param->generator == nullptr || param->generator == generator);
+            param->generator = generator;
+            filter_generator_params.push_back(param);
+            break;
+        }
+        case ObjectInstanceRegistry::GeneratorInput: {
+            if (outputs_seen) {
+                if (!generator->allow_out_of_order_inputs_and_outputs()) {
+                    user_warning << io_order_warning;
+                }
+            }
+            auto *input = static_cast<Internal::GeneratorInputBase *>(instance);
+            internal_assert(input != nullptr);
+            user_assert(is_valid_name(input->name())) << "Invalid Input name: (" << input->name() << ")\n";
+            user_assert(!names.count(input->name())) << "Duplicate Input name: " << input->name();
+            names.insert(input->name());
+            internal_assert(input->generator == nullptr || input->generator == generator);
+            input->generator = generator;
+            filter_inputs.push_back(input);
+            add_synthetic_params(input);
+            break;
+        }
+        case ObjectInstanceRegistry::GeneratorOutput: {
+            outputs_seen = true;
+            auto *output = static_cast<Internal::GeneratorOutputBase *>(instance);
+            internal_assert(output != nullptr);
+            user_assert(is_valid_name(output->name())) << "Invalid Output name: (" << output->name() << ")\n";
+            user_assert(!names.count(output->name())) << "Duplicate Output name: " << output->name();
+            names.insert(output->name());
+            internal_assert(output->generator == nullptr || output->generator == generator);
+            output->generator = generator;
+            filter_outputs.push_back(output);
+            add_synthetic_params(output);
+            break;
+        }
+        case ObjectInstanceRegistry::Generator:
+            // nothing
+            break;
+        case ObjectInstanceRegistry::FilterParam:
+            user_error << "ImageParam and Param<> are no longer allowed in Generators; use Input<> instead.";
+            break;
+        default:
+            user_error << "Unexpected ObjectInstanceRegistry::Kind value in GeneratorParamInfo. " << (int)kind;
+            break;
+        }
     }
 
     for (auto &g : owned_synthetic_params) {
@@ -1510,6 +1559,10 @@ std::vector<AbstractGenerator::ArgInfo> GeneratorBase::arginfos() {
     return args;
 }
 
+bool GeneratorBase::allow_out_of_order_inputs_and_outputs() const {
+    return false;
+}
+
 std::vector<Parameter> GeneratorBase::input_parameter(const std::string &name) {
     auto *input = find_input_by_name(name);
 
@@ -1586,6 +1639,26 @@ bool GeneratorBase::emit_cpp_stub(const std::string &stub_file_path) {
     return true;
 }
 
+bool GeneratorBase::emit_hlpipe(const std::string &hlpipe_file_path) {
+#ifdef WITH_SERIALIZATION
+    user_assert(!generator_registered_name.empty() && !generator_stub_name.empty()) << "Generator has no name.\n";
+    Pipeline pipeline = build_pipeline();
+    AutoSchedulerResults auto_schedule_results;
+    const auto context = this->context();
+    const auto &asp = context.autoscheduler_params();
+    if (!asp.name.empty()) {
+        debug(1) << "Applying autoscheduler " << asp.name << " to Generator " << name() << " ...\n";
+        auto_schedule_results = pipeline.apply_autoscheduler(context.target(), asp);
+    }
+    std::map<std::string, Parameter> params;  // FIXME: Remove when API allows this to be optional
+    serialize_pipeline(pipeline, hlpipe_file_path, params);
+    return true;
+#else
+    user_error << "Serialization is not supported in this build of Halide; try rebuilding with WITH_SERIALIZATION=ON.";
+    return false;
+#endif
+}
+
 GIOBase::GIOBase(size_t array_size,
                  const std::string &name,
                  ArgInfoKind kind,
@@ -2104,11 +2177,11 @@ void generator_test() {
         public:
             Input<int[]> expr_array_input{"expr_array_input"};
             Input<Func[]> func_array_input{"input_func_array"};
-            Input<Buffer<>[]> buffer_array_input { "buffer_array_input" };
+            Input<Buffer<>[]> buffer_array_input{"buffer_array_input"};
 
             Input<int[]> expr_array_output{"expr_array_output"};
             Output<Func[]> func_array_output{"func_array_output"};
-            Output<Buffer<>[]> buffer_array_output { "buffer_array_output" };
+            Output<Buffer<>[]> buffer_array_output{"buffer_array_output"};
 
             void generate() {
             }
