@@ -342,7 +342,11 @@ Stmt ProducerConsumer::make_consume(const std::string &name, Stmt body) {
     return ProducerConsumer::make(name, false, std::move(body));
 }
 
-Stmt For::make(const std::string &name, Expr min, Expr extent, ForType for_type, DeviceAPI device_api, Stmt body) {
+Stmt For::make(const std::string &name,
+               Expr min, Expr extent,
+               ForType for_type, Partition partition_policy,
+               DeviceAPI device_api,
+               Stmt body) {
     internal_assert(min.defined()) << "For of undefined\n";
     internal_assert(extent.defined()) << "For of undefined\n";
     internal_assert(min.type() == Int(32)) << "For with non-integer min\n";
@@ -354,6 +358,7 @@ Stmt For::make(const std::string &name, Expr min, Expr extent, ForType for_type,
     node->min = std::move(min);
     node->extent = std::move(extent);
     node->for_type = for_type;
+    node->partition_policy = partition_policy;
     node->device_api = device_api;
     node->body = std::move(body);
     return node;
@@ -574,6 +579,8 @@ Stmt IfThenElse::make(Expr condition, Stmt then_case, Stmt else_case) {
     internal_assert(condition.defined() && then_case.defined()) << "IfThenElse of undefined\n";
     // else_case may be null.
 
+    internal_assert(condition.type().is_scalar()) << "IfThenElse with vector condition\n";
+
     IfThenElse *node = new IfThenElse;
     node->condition = std::move(condition);
     node->then_case = std::move(then_case);
@@ -667,6 +674,7 @@ const char *const intrinsic_op_names[] = {
     "shift_right",
     "signed_integer_overflow",
     "size_of_halide_buffer_t",
+    "sliding_window_marker",
     "sorted_avg",
     "strict_float",
     "stringify",
@@ -911,6 +919,15 @@ Stmt Atomic::make(const std::string &producer_name,
     return node;
 }
 
+Stmt HoistedStorage::make(const std::string &name,
+                          Stmt body) {
+    internal_assert(body.defined()) << "HoistedStorage must have a body statement.\n";
+    HoistedStorage *node = new HoistedStorage;
+    node->name = name;
+    node->body = std::move(body);
+    return node;
+}
+
 Expr VectorReduce::make(VectorReduce::Operator op,
                         Expr vec,
                         int lanes) {
@@ -1162,6 +1179,10 @@ template<>
 void StmtNode<Atomic>::accept(IRVisitor *v) const {
     v->visit((const Atomic *)this);
 }
+template<>
+void StmtNode<HoistedStorage>::accept(IRVisitor *v) const {
+    v->visit((const HoistedStorage *)this);
+}
 
 template<>
 Expr ExprNode<IntImm>::mutate_expr(IRMutator *v) const {
@@ -1352,6 +1373,10 @@ template<>
 Stmt StmtNode<Atomic>::mutate_stmt(IRMutator *v) const {
     return v->visit((const Atomic *)this);
 }
+template<>
+Stmt StmtNode<HoistedStorage>::mutate_stmt(IRMutator *v) const {
+    return v->visit((const HoistedStorage *)this);
+}
 
 Call::ConstString Call::buffer_get_dimensions = "_halide_buffer_get_dimensions";
 Call::ConstString Call::buffer_get_min = "_halide_buffer_get_min";
