@@ -7,6 +7,7 @@
 
 #include "CSE.h"
 #include "Debug.h"
+#include "Func.h"
 #include "IREquality.h"
 #include "IRMutator.h"
 #include "IROperator.h"
@@ -1502,8 +1503,12 @@ Expr select(Expr condition, Expr true_value, Expr false_value) {
 }
 
 Tuple tuple_select(const Tuple &condition, const Tuple &true_value, const Tuple &false_value) {
+    return select(condition, true_value, false_value);
+}
+
+Tuple select(const Tuple &condition, const Tuple &true_value, const Tuple &false_value) {
     user_assert(condition.size() == true_value.size() && true_value.size() == false_value.size())
-        << "tuple_select() requires all Tuples to have identical sizes.";
+        << "select() on Tuples requires all Tuples to have identical sizes.";
     Tuple result(std::vector<Expr>(condition.size()));
     for (size_t i = 0; i < result.size(); i++) {
         result[i] = select(condition[i], true_value[i], false_value[i]);
@@ -1512,8 +1517,12 @@ Tuple tuple_select(const Tuple &condition, const Tuple &true_value, const Tuple
 }
 
 Tuple tuple_select(const Expr &condition, const Tuple &true_value, const Tuple &false_value) {
+    return select(condition, true_value, false_value);
+}
+
+Tuple select(const Expr &condition, const Tuple &true_value, const Tuple &false_value) {
     user_assert(true_value.size() == false_value.size())
-        << "tuple_select() requires all Tuples to have identical sizes.";
+        << "select() on Tuples requires all Tuples to have identical sizes.";
     Tuple result(std::vector<Expr>(true_value.size()));
     for (size_t i = 0; i < result.size(); i++) {
         result[i] = select(condition, true_value[i], false_value[i]);
@@ -1521,6 +1530,10 @@ Tuple tuple_select(const Expr &condition, const Tuple &true_value, const Tuple &
     return result;
 }
 
+Expr select(const Expr &condition, const FuncRef &true_value, const FuncRef &false_value) {
+    return select(condition, (Expr)true_value, (Expr)false_value);
+}
+
 Expr mux(const Expr &id, const std::vector<Expr> &values) {
     user_assert(!values.empty()) << "mux() requires a non-empty vector of values";
     if (values.size() == 1) {
@@ -1547,13 +1560,37 @@ Expr mux(const Expr &id, const std::initializer_list<Expr> &values) {
     return mux(id, std::vector<Expr>(values));
 }
 
+Expr mux(const Expr &id, const std::initializer_list<FuncRef> &values) {
+    std::vector<Expr> exprs(values.size());
+    for (size_t i = 0; i < values.size(); i++) {
+        exprs[i] = Expr(*(values.begin() + i));
+    }
+    return mux(id, exprs);
+}
+
+Tuple mux(const Expr &id, const std::initializer_list<Tuple> &values) {
+    return mux(id, std::vector<Tuple>(values));
+}
+
+Tuple mux(const Expr &id, const std::vector<Tuple> &values) {
+    user_assert(!values.empty()) << "mux() requires a non-empty vector of values";
+    std::vector<Expr> result(values[0].size());
+    for (size_t i = 0; i < result.size(); i++) {
+        std::vector<Expr> elems(values.size());
+        for (size_t j = 0; j < values.size(); j++) {
+            elems[j] = values[j][i];
+        }
+        result[i] = mux(id, elems);
+    }
+    return Tuple{result};
+}
+
 Expr unsafe_promise_clamped(const Expr &value, const Expr &min, const Expr &max) {
     user_assert(value.defined()) << "unsafe_promise_clamped with undefined value.\n";
     Expr n_min_val = min.defined() ? lossless_cast(value.type(), min) : value.type().min();
     Expr n_max_val = max.defined() ? lossless_cast(value.type(), max) : value.type().max();
 
     // Min and max are allowed to be undefined with the meaning of no bound on that side.
-
     return Call::make(value.type(),
                       Call::unsafe_promise_clamped,
                       {value, n_min_val, n_max_val},
@@ -2556,10 +2593,23 @@ Expr mod_round_to_zero(Expr x, Expr y) {
                       Call::PureIntrinsic);
 }
 
+namespace {
+
+std::atomic<int> random_number_counter = 0;
+
+}  // namespace
+
+namespace Internal {
+
+void reset_random_counters() {
+    random_number_counter = 0;
+    random_variable_counter = 0;
+}
+
+}  // namespace Internal
+
 Expr random_float(Expr seed) {
-    // Random floats get even IDs
-    static std::atomic<int> counter{0};
-    int id = (counter++) * 2;
+    const int id = random_number_counter++;
 
     std::vector<Expr> args;
     if (seed.defined()) {
@@ -2577,9 +2627,7 @@ Expr random_float(Expr seed) {
 }
 
 Expr random_uint(Expr seed) {
-    // Random ints get odd IDs
-    static std::atomic<int> counter{0};
-    int id = (counter++) * 2 + 1;
+    const int id = random_number_counter++;
 
     std::vector<Expr> args;
     if (seed.defined()) {
@@ -2595,7 +2643,7 @@ Expr random_uint(Expr seed) {
 }
 
 Expr random_int(Expr seed) {
-    return cast<int32_t>(random_uint(std::move(seed)));
+    return reinterpret<int32_t>(random_uint(std::move(seed)));
 }
 
 Expr likely(Expr e) {
