@@ -351,6 +351,12 @@ Expr requirement_failed_error(Expr condition, const std::vector<Expr> &args);
 
 Expr memoize_tag_helper(Expr result, const std::vector<Expr> &cache_key_values);
 
+/** Reset the counters used for random-number seeds in random_float/int/uint.
+ * (Note that the counters are incremented for each call, even if a seed is passed in.)
+ * This is used for multitarget compilation to ensure that each subtarget gets
+ * the same sequence of random numbers. */
+void reset_random_counters();
+
 }  // namespace Internal
 
 /** Cast an expression to the halide type corresponding to the C++ type T. */
@@ -795,22 +801,47 @@ inline Expr select(Expr c0, Expr v0, Expr c1, Expr v1, Args &&...args) {
 /** Equivalent of ternary select(), but taking/returning tuples. If the condition is
  * a Tuple, it must match the size of the true and false Tuples. */
 // @{
+HALIDE_ATTRIBUTE_DEPRECATED("tuple_select has been deprecated. Use select instead (which now works for Tuples)")
 Tuple tuple_select(const Tuple &condition, const Tuple &true_value, const Tuple &false_value);
+HALIDE_ATTRIBUTE_DEPRECATED("tuple_select has been deprecated. Use select instead (which now works for Tuples)")
 Tuple tuple_select(const Expr &condition, const Tuple &true_value, const Tuple &false_value);
+Tuple select(const Tuple &condition, const Tuple &true_value, const Tuple &false_value);
+Tuple select(const Expr &condition, const Tuple &true_value, const Tuple &false_value);
 // @}
 
 /** Equivalent of multiway select(), but taking/returning tuples. If the condition is
  * a Tuple, it must match the size of the true and false Tuples. */
 // @{
 template<typename... Args>
+HALIDE_ATTRIBUTE_DEPRECATED("tuple_select has been deprecated. Use select instead (which now works for Tuples)")
 inline Tuple tuple_select(const Tuple &c0, const Tuple &v0, const Tuple &c1, const Tuple &v1, Args &&...args) {
     return tuple_select(c0, v0, tuple_select(c1, v1, std::forward<Args>(args)...));
 }
-
 template<typename... Args>
+HALIDE_ATTRIBUTE_DEPRECATED("tuple_select has been deprecated. Use select instead (which now works for Tuples)")
 inline Tuple tuple_select(const Expr &c0, const Tuple &v0, const Expr &c1, const Tuple &v1, Args &&...args) {
     return tuple_select(c0, v0, tuple_select(c1, v1, std::forward<Args>(args)...));
 }
+template<typename... Args>
+inline Tuple select(const Tuple &c0, const Tuple &v0, const Tuple &c1, const Tuple &v1, Args &&...args) {
+    return select(c0, v0, select(c1, v1, std::forward<Args>(args)...));
+}
+template<typename... Args>
+inline Tuple select(const Expr &c0, const Tuple &v0, const Expr &c1, const Tuple &v1, Args &&...args) {
+    return select(c0, v0, select(c1, v1, std::forward<Args>(args)...));
+}
+// @}
+
+/** select applied to FuncRefs (e.g. select(x < 100, f(x), g(x))) is assumed to
+ * return an Expr. A runtime error is produced if this is applied to
+ * tuple-valued Funcs. In that case you should explicitly cast the second and
+ * third args to Tuple to remove the ambiguity. */
+// @{
+Expr select(const Expr &condition, const FuncRef &true_value, const FuncRef &false_value);
+template<typename... Args>
+inline Expr select(const Expr &c0, const FuncRef &v0, const Expr &c1, const FuncRef &v1, Args &&...args) {
+    return select(c0, v0, select(c1, v1, std::forward<Args>(args)...));
+}
 // @}
 
 /** Oftentimes we want to pack a list of expressions with the same type
@@ -829,6 +860,9 @@ inline Tuple tuple_select(const Expr &c0, const Tuple &v0, const Expr &c1, const
 Expr mux(const Expr &id, const std::initializer_list<Expr> &values);
 Expr mux(const Expr &id, const std::vector<Expr> &values);
 Expr mux(const Expr &id, const Tuple &values);
+Expr mux(const Expr &id, const std::initializer_list<FuncRef> &values);
+Tuple mux(const Expr &id, const std::initializer_list<Tuple> &values);
+Tuple mux(const Expr &id, const std::vector<Tuple> &values);
 // @}
 
 /** Return the sine of a floating-point expression. If the argument is
