@@ -334,7 +334,11 @@ void JITModule::compile_module(std::unique_ptr<llvm::Module> m, const string &fu
     // Build TargetMachine
     llvm::orc::JITTargetMachineBuilder tm_builder(llvm::Triple(m->getTargetTriple()));
     tm_builder.setOptions(options);
+#if LLVM_VERSION >= 180
+    tm_builder.setCodeGenOptLevel(CodeGenOptLevel::Aggressive);
+#else
     tm_builder.setCodeGenOptLevel(CodeGenOpt::Aggressive);
+#endif
     if (target.arch == Target::Arch::RISCV) {
         tm_builder.setCodeModel(llvm::CodeModel::Medium);
     }
@@ -520,7 +524,9 @@ JITModule::Symbol JITModule::argv_entrypoint_symbol() const {
     return jit_module->argv_entrypoint;
 }
 
-static bool module_already_in_graph(const JITModuleContents *start, const JITModuleContents *target, std::set<const JITModuleContents *> &already_seen) {
+namespace {
+
+bool module_already_in_graph(const JITModuleContents *start, const JITModuleContents *target, std::set<const JITModuleContents *> &already_seen) {
     if (start == target) {
         return true;
     }
@@ -537,6 +543,8 @@ static bool module_already_in_graph(const JITModuleContents *start, const JITMod
     return false;
 }
 
+}  // namespace
+
 void JITModule::add_dependency(JITModule &dep) {
     std::set<const JITModuleContents *> already_seen;
     internal_assert(!module_already_in_graph(dep.jit_module.get(), jit_module.get(), already_seen)) << "JITModule::add_dependency: creating circular dependency graph.\n";
