@@ -319,7 +319,7 @@ class LICM : public IRMutator {
             internal_assert(loop);
 
             new_stmt = For::make(loop->name, loop->min, loop->extent,
-                                 loop->for_type, loop->device_api, mutate(loop->body));
+                                 loop->for_type, loop->partition_policy, loop->device_api, mutate(loop->body));
 
             // Wrap lets for the lifted invariants
             for (size_t i = 0; i < exprs.size(); i++) {
@@ -564,7 +564,7 @@ class HoistIfStatements : public IRMutator {
                 is_pure(i->condition) &&
                 !expr_uses_var(i->condition, op->name)) {
                 Stmt s = For::make(op->name, op->min, op->extent,
-                                   op->for_type, op->device_api, i->then_case);
+                                   op->for_type, op->partition_policy, op->device_api, i->then_case);
                 return IfThenElse::make(i->condition, s);
             }
         }
@@ -572,7 +572,7 @@ class HoistIfStatements : public IRMutator {
             return op;
         } else {
             return For::make(op->name, op->min, op->extent,
-                             op->for_type, op->device_api, body);
+                             op->for_type, op->partition_policy, op->device_api, body);
         }
     }
 
