@@ -389,13 +389,17 @@ void emit_file(const llvm::Module &module_in, Internal::LLVMOStream &out,
     // in Codegen_LLVM.cpp, but since this seems to be have added in error,
     // we're just going to elide it for LLVM >= 17.0
     pass_manager.add(llvm::createStripDeadDebugInfoPass());
-#endif
 
     // Enable symbol rewriting. This allows code outside libHalide to
     // use symbol rewriting when compiling Halide code (for example, by
     // using cl::ParseCommandLineOption and then passing the appropriate
     // rewrite options via -mllvm flags).
     pass_manager.add(llvm::createRewriteSymbolsPass());
+#endif
+
+    if (target_machine->isPositionIndependent()) {
+        Internal::debug(1) << "Target machine is Position Independent!\n";
+    }
 
     // Override default to generate verbose assembly.
     target_machine->Options.MCOptions.AsmVerbose = true;
@@ -421,11 +425,19 @@ std::unique_ptr<llvm::Module> compile_module_to_llvm_module(const Module &module
 }
 
 void compile_llvm_module_to_object(llvm::Module &module, Internal::LLVMOStream &out) {
+#if LLVM_VERSION >= 180
+    emit_file(module, out, llvm::CodeGenFileType::ObjectFile);
+#else
     emit_file(module, out, llvm::CGFT_ObjectFile);
+#endif
 }
 
 void compile_llvm_module_to_assembly(llvm::Module &module, Internal::LLVMOStream &out) {
+#if LLVM_VERSION >= 180
+    emit_file(module, out, llvm::CodeGenFileType::AssemblyFile);
+#else
     emit_file(module, out, llvm::CGFT_AssemblyFile);
+#endif
 }
 
 void compile_llvm_module_to_llvm_bitcode(llvm::Module &module, Internal::LLVMOStream &out) {
@@ -593,7 +605,11 @@ void create_static_library(const std::vector<std::string> &src_files_in, const T
         return;
     }
 
+#if LLVM_VERSION >= 180
+    const llvm::SymtabWritingMode write_symtab = llvm::SymtabWritingMode::NormalSymtab;
+#else
     const bool write_symtab = true;
+#endif
     const auto kind = Internal::get_triple_for_target(target).isOSDarwin() ? llvm::object::Archive::K_BSD : llvm::object::Archive::K_GNU;
     const bool thin = false;
     auto result = llvm::writeArchive(dst_file, new_members,
