@@ -11,6 +11,7 @@
 #include "AddParameterChecks.h"
 #include "AllocationBoundsInference.h"
 #include "AsyncProducers.h"
+#include "BoundConstantExtentLoops.h"
 #include "BoundSmallAllocations.h"
 #include "Bounds.h"
 #include "BoundsInference.h"
@@ -312,6 +313,10 @@ void lower_impl(const vector<Function> &output_funcs,
     s = simplify_correlated_differences(s);
     log("Lowering after simplifying correlated differences:", s);
 
+    debug(1) << "Bounding constant extent loops...\n";
+    s = bound_constant_extent_loops(s);
+    log("Lowering after bounding constant extent loops:", s);
+
     debug(1) << "Unrolling...\n";
     s = unroll_loops(s);
     log("Lowering after unrolling:", s);
@@ -432,6 +437,9 @@ void lower_impl(const vector<Function> &output_funcs,
         }
     }
 
+    // Make a copy of the Stmt code, before we lower anything to less human-readable code.
+    result_module.set_conceptual_code_stmt(s);
+
     if (t.arch != Target::Hexagon && t.has_feature(Target::HVX)) {
         debug(1) << "Splitting off Hexagon offload...\n";
         s = inject_hexagon_rpc(s, t, result_module);
