@@ -1,5 +1,4 @@
 #include <algorithm>
-#include <numeric>
 #include <utility>
 
 #include "CSE.h"
@@ -89,13 +88,22 @@ protected:
         }
     }
 
+    void visit(const Variable *op) override {
+        result |= scope.contains(op->name);
+    }
+
+    const Scope<> &scope;
+
 public:
+    HasLikelyTag(const Scope<> &s)
+        : scope(s) {
+    }
+
     bool result = false;
 };
 
 class HasUncapturedLikelyTag : public HasLikelyTag {
     using HasLikelyTag::visit;
-
     // Any likelies buried inside the following ops are captured the by respective ops
     void visit(const Select *op) override {
     }
@@ -103,6 +111,11 @@ class HasUncapturedLikelyTag : public HasLikelyTag {
     }
     void visit(const Max *op) override {
     }
+
+public:
+    HasUncapturedLikelyTag(const Scope<> &s)
+        : HasLikelyTag(s) {
+    }
 };
 
 // The goal of loop partitioning is to split loops up into a prologue,
@@ -227,9 +240,9 @@ class ExprUsesInvalidBuffers : public IRVisitor {
 
 public:
     ExprUsesInvalidBuffers(const Scope<> &buffers)
-        : invalid_buffers(buffers), invalid(false) {
+        : invalid_buffers(buffers) {
     }
-    bool invalid;
+    bool invalid = false;
 };
 
 /** Check if any references to buffers in an expression is invalid. */
@@ -244,6 +257,7 @@ class FindSimplifications : public IRVisitor {
     using IRVisitor::visit;
 
     Scope<> depends_on_loop_var, depends_on_invalid_buffers;
+    Scope<> vars_with_uncaptured_likely, vars_with_likely;
     Scope<> buffers;
 
     void visit(const Allocate *op) override {
@@ -264,23 +278,20 @@ class FindSimplifications : public IRVisitor {
         }
         condition = remove_likelies(condition);
         Simplification s = {condition, std::move(old), std::move(likely_val), std::move(unlikely_val), true};
-        if (s.condition.type().is_vector()) {
-            s.condition = simplify(s.condition);
-            if (const Broadcast *b = s.condition.as<Broadcast>()) {
-                s.condition = b->value;
-            } else {
-                // Devectorize the condition
-                s.condition = and_condition_over_domain(s.condition, Scope<Interval>::empty_scope());
-                s.tight = false;
-            }
-        }
-        internal_assert(s.condition.type().is_scalar()) << s.condition << "\n";
         simplifications.push_back(s);
     }
 
+    bool has_uncaptured_likely(const Expr &e) const {
+        return has_uncaptured_likely_tag(e, vars_with_uncaptured_likely);
+    }
+
+    bool has_likely(const Expr &e) const {
+        return has_likely_tag(e, vars_with_likely);
+    }
+
     void visit(const Min *op) override {
-        bool likely_a = has_uncaptured_likely_tag(op->a);
-        bool likely_b = has_uncaptured_likely_tag(op->b);
+        bool likely_a = has_uncaptured_likely(op->a);
+        bool likely_b = has_uncaptured_likely(op->b);
 
         // If one side has an uncaptured likely, don't hunt for
         // simplifications in the other side.
@@ -295,20 +306,23 @@ class FindSimplifications : public IRVisitor {
         // call. If neither does, prefer the side that contains any
         // likely call at all.
         if (!likely_a && !likely_b) {
-            likely_a = has_likely_tag(op->a);
-            likely_b = has_likely_tag(op->b);
+            likely_a = has_likely(op->a);
+            likely_b = has_likely(op->b);
         }
 
         if (likely_b && !likely_a) {
             new_simplification(op->b <= op->a, op, op->b, op->a);
         } else if (likely_a && !likely_b) {
             new_simplification(op->a <= op->b, op, op->a, op->b);
+        } else if (likely_a && likely_b) {
+            // Likelies on both sides, continue inwards.
+            IRVisitor::visit(op);
         }
     }
 
     void visit(const Max *op) override {
-        bool likely_a = has_uncaptured_likely_tag(op->a);
-        bool likely_b = has_uncaptured_likely_tag(op->b);
+        bool likely_a = has_uncaptured_likely(op->a);
+        bool likely_b = has_uncaptured_likely(op->b);
 
         if (!likely_a) {
             op->b.accept(this);
@@ -318,8 +332,8 @@ class FindSimplifications : public IRVisitor {
         }
 
         if (!likely_a && !likely_b) {
-            likely_a = has_likely_tag(op->a);
-            likely_b = has_likely_tag(op->b);
+            likely_a = has_likely(op->a);
+            likely_b = has_likely(op->b);
         }
 
         if (likely_b && !likely_a) {
@@ -332,13 +346,8 @@ class FindSimplifications : public IRVisitor {
     void visit_select(const Expr &condition, const Expr &old, const Expr &true_value, const Expr &false_value) {
         condition.accept(this);
 
-        bool likely_t = has_uncaptured_likely_tag(true_value);
-        bool likely_f = has_uncaptured_likely_tag(false_value);
-
-        if (!likely_t && !likely_f) {
-            likely_t = has_likely_tag(true_value);
-            likely_f = has_likely_tag(false_value);
-        }
+        bool likely_t = has_uncaptured_likely(true_value);
+        bool likely_f = has_uncaptured_likely(false_value);
 
         if (!likely_t) {
             false_value.accept(this);
@@ -347,6 +356,11 @@ class FindSimplifications : public IRVisitor {
             true_value.accept(this);
         }
 
+        if (!likely_t && !likely_f) {
+            likely_t = has_likely(true_value);
+            likely_f = has_likely(false_value);
+        }
+
         if (likely_t && !likely_f) {
             new_simplification(condition, old, true_value, false_value);
         } else if (likely_f && !likely_t) {
@@ -377,7 +391,7 @@ class FindSimplifications : public IRVisitor {
         // statement is marked as likely, treat it as likely true and
         // partition accordingly.
         IRVisitor::visit(op);
-        if (has_uncaptured_likely_tag(op->condition)) {
+        if (has_uncaptured_likely(op->condition)) {
             new_simplification(op->condition, op->condition, const_true(), const_false());
         }
     }
@@ -409,17 +423,17 @@ class FindSimplifications : public IRVisitor {
 
     void visit(const Store *op) override {
         IRVisitor::visit(op);
-        if (has_uncaptured_likely_tag(op->predicate)) {
+        if (has_uncaptured_likely(op->predicate)) {
             const int lanes = op->predicate.type().lanes();
-            new_simplification(op->predicate, op->predicate, const_true(lanes), op->predicate);
+            new_simplification(op->predicate, op->predicate, const_true(lanes), remove_likelies(op->predicate));
         }
     }
 
     void visit(const Load *op) override {
         IRVisitor::visit(op);
-        if (has_uncaptured_likely_tag(op->predicate)) {
+        if (has_uncaptured_likely(op->predicate)) {
             const int lanes = op->predicate.type().lanes();
-            new_simplification(op->predicate, op->predicate, const_true(lanes), op->predicate);
+            new_simplification(op->predicate, op->predicate, const_true(lanes), remove_likelies(op->predicate));
         }
     }
 
@@ -430,6 +444,11 @@ class FindSimplifications : public IRVisitor {
         ScopedBinding<> bind_invalid(expr_uses_invalid_buffers(op->value, buffers) ||
                                          expr_uses_vars(op->value, depends_on_invalid_buffers),
                                      depends_on_invalid_buffers, op->name);
+        ScopedBinding<> bind_uncaptured_likely(has_uncaptured_likely(op->value),
+                                               vars_with_uncaptured_likely, op->name);
+        ScopedBinding<> bind_likely(has_likely(op->value),
+                                    vars_with_likely, op->name);
+
         vector<Simplification> old;
         old.swap(simplifications);
         IRVisitor::visit(op);
@@ -472,6 +491,9 @@ public:
     Expr mutate(const Expr &e) override {
         for (auto const &s : simplifications) {
             if (e.same_as(s.old_expr)) {
+                internal_assert(!s.likely_value.same_as(s.old_expr))
+                    << "Loop partitioning simplification does not mutate value: "
+                    << s.old_expr << "\n";
                 return mutate(s.likely_value);
             }
         }
@@ -515,10 +537,35 @@ class PartitionLoops : public IRMutator {
     using IRMutator::visit;
 
     bool in_gpu_loop = false;
+    bool in_tail = false;
 
     Stmt visit(const For *op) override {
+        // Do not partition if the schedule explicitly forbids, or if it's set
+        // to automatic and we're in a loop tail.
+        if (op->partition_policy == Partition::Never ||
+            (op->partition_policy == Partition::Auto && in_tail)) {
+            return IRMutator::visit(op);
+        }
+
         Stmt body = op->body;
 
+        // A struct that upon destruction will check if the current For was partitioned
+        // and error out if it wasn't when the schedule demanded it.
+        struct ErrorIfNotMutated {
+            const For *op;
+            bool must_mutate;
+            bool mutated{false};
+            ErrorIfNotMutated(const For *op, bool must_mutate)
+                : op(op), must_mutate(must_mutate) {
+            }
+            ~ErrorIfNotMutated() {
+                if (must_mutate && !mutated) {
+                    user_error << "Loop Partition Policy is set to " << op->partition_policy
+                               << " for " << op->name << ", but no loop partitioning was performed.";
+                }
+            }
+        } mutation_checker{op, op->partition_policy == Partition::Always};
+
         ScopedValue<bool> old_in_gpu_loop(in_gpu_loop, in_gpu_loop ||
                                                            CodeGen_GPU_Dev::is_gpu_var(op->name));
 
@@ -542,6 +589,18 @@ class PartitionLoops : public IRMutator {
         vector<Simplification> middle_simps, prologue_simps, epilogue_simps;
         bool lower_bound_is_tight = true, upper_bound_is_tight = true;
         for (auto &s : finder.simplifications) {
+
+            // Devectorize the condition
+            while (s.condition.type().is_vector()) {
+                s.condition = simplify(s.condition);
+                if (const Broadcast *b = s.condition.as<Broadcast>()) {
+                    s.condition = b->value;
+                } else {
+                    s.condition = and_condition_over_domain(s.condition, Scope<Interval>::empty_scope());
+                    s.tight = false;
+                }
+            }
+
             // Solve for the interval over which this simplification is true.
             s.interval = solve_for_inner_interval(s.condition, op->name);
             if (s.tight) {
@@ -663,6 +722,13 @@ class PartitionLoops : public IRMutator {
         // Recurse on the middle section.
         simpler_body = mutate(simpler_body);
 
+        // Recurse on the prologue and epilogue, just for loops set to Partition::Always
+        {
+            ScopedValue<bool> s(in_tail, true);
+            epilogue = mutate(epilogue);
+            prologue = mutate(prologue);
+        }
+
         // Construct variables for the bounds of the simplified middle section
         Expr min_steady = op->min, max_steady = op->extent + op->min;
         Expr prologue_val, epilogue_val;
@@ -704,17 +770,19 @@ class PartitionLoops : public IRMutator {
         // Bust simple serial for loops up into three.
         if (op->for_type == ForType::Serial && !op->body.as<Acquire>()) {
             stmt = For::make(op->name, min_steady, max_steady - min_steady,
-                             op->for_type, op->device_api, simpler_body);
+                             op->for_type, op->partition_policy, op->device_api, simpler_body);
 
             if (make_prologue) {
                 prologue = For::make(op->name, op->min, min_steady - op->min,
-                                     op->for_type, op->device_api, prologue);
+                                     op->for_type, op->partition_policy, op->device_api, prologue);
                 stmt = Block::make(prologue, stmt);
+                mutation_checker.mutated = true;
             }
             if (make_epilogue) {
                 epilogue = For::make(op->name, max_steady, op->min + op->extent - max_steady,
-                                     op->for_type, op->device_api, epilogue);
+                                     op->for_type, op->partition_policy, op->device_api, epilogue);
                 stmt = Block::make(stmt, epilogue);
+                mutation_checker.mutated = true;
             }
         } else {
             // For parallel for loops we could use a Fork node here,
@@ -733,15 +801,18 @@ class PartitionLoops : public IRMutator {
             stmt = simpler_body;
             if (make_epilogue && make_prologue && equal(prologue, epilogue)) {
                 stmt = IfThenElse::make(min_steady <= loop_var && loop_var < max_steady, stmt, prologue);
+                mutation_checker.mutated = true;
             } else {
                 if (make_epilogue) {
                     stmt = IfThenElse::make(loop_var < max_steady, stmt, epilogue);
+                    mutation_checker.mutated = true;
                 }
                 if (make_prologue) {
                     stmt = IfThenElse::make(loop_var < min_steady, prologue, stmt);
+                    mutation_checker.mutated = true;
                 }
             }
-            stmt = For::make(op->name, op->min, op->extent, op->for_type, op->device_api, stmt);
+            stmt = For::make(op->name, op->min, op->extent, op->for_type, op->partition_policy, op->device_api, stmt);
         }
 
         if (make_epilogue) {
@@ -864,7 +935,7 @@ class RenormalizeGPULoops : public IRMutator {
             internal_assert(!expr_uses_var(f->min, op->name) &&
                             !expr_uses_var(f->extent, op->name));
             Stmt inner = LetStmt::make(op->name, op->value, f->body);
-            inner = For::make(f->name, f->min, f->extent, f->for_type, f->device_api, inner);
+            inner = For::make(f->name, f->min, f->extent, f->for_type, f->partition_policy, f->device_api, inner);
             return mutate(inner);
         } else if (a && in_gpu_loop && !in_thread_loop) {
             internal_assert(a->extents.size() == 1);
@@ -942,7 +1013,7 @@ class RenormalizeGPULoops : public IRMutator {
                    for_a->min.same_as(for_b->min) &&
                    for_a->extent.same_as(for_b->extent)) {
             Stmt inner = IfThenElse::make(op->condition, for_a->body, for_b->body);
-            inner = For::make(for_a->name, for_a->min, for_a->extent, for_a->for_type, for_a->device_api, inner);
+            inner = For::make(for_a->name, for_a->min, for_a->extent, for_a->for_type, for_a->partition_policy, for_a->device_api, inner);
             return mutate(inner);
         } else {
             internal_error << "Unexpected construct inside if statement: " << Stmt(op) << "\n";
@@ -962,24 +1033,44 @@ class ExpandSelects : public IRMutator {
 
     Expr visit(const Select *op) override {
         Expr condition = mutate(op->condition);
+
+        const Call *true_likely = Call::as_intrinsic(op->true_value, {Call::likely});
+        const Call *false_likely = Call::as_intrinsic(op->false_value, {Call::likely});
+
         Expr true_value = mutate(op->true_value);
         Expr false_value = mutate(op->false_value);
         if (const Or *o = condition.as<Or>()) {
             if (is_trivial(true_value)) {
-                return mutate(Select::make(o->a, true_value, Select::make(o->b, true_value, false_value)));
+                Expr expr = Select::make(o->b, true_value, false_value);
+                if (false_likely) {
+                    expr = likely(expr);
+                }
+                return mutate(Select::make(o->a, true_value, expr));
             } else {
                 string var_name = unique_name('t');
                 Expr var = Variable::make(true_value.type(), var_name);
-                Expr expr = mutate(Select::make(o->a, var, Select::make(o->b, var, false_value)));
+                Expr expr = Select::make(o->b, var, false_value);
+                if (false_likely) {
+                    expr = likely(expr);
+                }
+                expr = mutate(Select::make(o->a, var, expr));
                 return Let::make(var_name, true_value, expr);
             }
         } else if (const And *a = condition.as<And>()) {
             if (is_trivial(false_value)) {
-                return mutate(Select::make(a->a, Select::make(a->b, true_value, false_value), false_value));
+                Expr expr = Select::make(a->b, true_value, false_value);
+                if (true_likely) {
+                    expr = likely(expr);
+                }
+                return mutate(Select::make(a->a, expr, false_value));
             } else {
                 string var_name = unique_name('t');
                 Expr var = Variable::make(false_value.type(), var_name);
-                Expr expr = mutate(Select::make(a->a, Select::make(a->b, true_value, var), var));
+                Expr expr = Select::make(a->b, true_value, var);
+                if (true_likely) {
+                    expr = likely(expr);
+                }
+                expr = mutate(Select::make(a->a, expr, var));
                 return Let::make(var_name, false_value, expr);
             }
         } else if (const Not *n = condition.as<Not>()) {
@@ -1069,14 +1160,14 @@ class LowerLikelyIfInnermost : public IRMutator {
 
 }  // namespace
 
-bool has_uncaptured_likely_tag(const Expr &e) {
-    HasUncapturedLikelyTag h;
+bool has_uncaptured_likely_tag(const Expr &e, const Scope<> &scope) {
+    HasUncapturedLikelyTag h(scope);
     e.accept(&h);
     return h.result;
 }
 
-bool has_likely_tag(const Expr &e) {
-    HasLikelyTag h;
+bool has_likely_tag(const Expr &e, const Scope<> &scope) {
+    HasLikelyTag h(scope);
     e.accept(&h);
     return h.result;
 }
