@@ -13,6 +13,7 @@
 #include "DeviceAPI.h"
 #include "Expr.h"
 #include "FunctionPtr.h"
+#include "LoopPartitioningDirective.h"
 #include "Parameter.h"
 #include "PrefetchDirective.h"
 
@@ -99,6 +100,32 @@ enum class TailStrategy {
      * instead of a multiple of the split factor as with RoundUp. */
     ShiftInwards,
 
+    /** Equivalent to ShiftInwards, but protects values that would be
+     * re-evaluated by loading the memory location that would be stored to,
+     * modifying only the elements not contained within the overlap, and then
+     * storing the blended result.
+     *
+     * This tail strategy is useful when you want to use ShiftInwards to
+     * vectorize without a scalar tail, but are scheduling a stage where that
+     * isn't legal (e.g. an update definition).
+     *
+     * Because this is a read - modify - write, this tail strategy cannot be
+     * used on any dimension the stage is parallelized over as it would cause a
+     * race condition.
+     */
+    ShiftInwardsAndBlend,
+
+    /** Equivalent to RoundUp, but protected values that would be written beyond
+     * the end by loading the memory location that would be stored to,
+     * modifying only the elements within the region being computed, and then
+     * storing the blended result.
+     *
+     * This tail strategy is useful when vectorizing an update to some sub-region
+     * of a larger Func. As with ShiftInwardsAndBlend, it can't be combined with
+     * parallelism.
+     */
+    RoundUpAndBlend,
+
     /** For pure definitions use ShiftInwards. For pure vars in
      * update definitions use RoundUp. For RVars in update
      * definitions use GuardWithIf. */
@@ -179,8 +206,6 @@ class LoopLevel {
     explicit LoopLevel(Internal::IntrusivePtr<Internal::LoopLevelContents> c)
         : contents(std::move(c)) {
     }
-    LoopLevel(const std::string &func_name, const std::string &var_name,
-              bool is_rvar, int stage_index, bool locked = false);
 
 public:
     /** Return the index of the function stage associated with this loop level.
@@ -197,6 +222,10 @@ public:
      * LoopLevel (other than set()) will assert. */
     LoopLevel();
 
+    /** For deserialization only. */
+    LoopLevel(const std::string &func_name, const std::string &var_name,
+              bool is_rvar, int stage_index, bool locked = false);
+
     /** Construct a special LoopLevel value that implies
      * that a function should be inlined away. */
     static LoopLevel inlined();
@@ -233,6 +262,21 @@ public:
     // outside of all for loops.
     bool is_root() const;
 
+    // For serialization only. Do not use in other cases.
+    int get_stage_index() const;
+
+    // For serialization only. Do not use in other cases.
+    std::string func_name() const;
+
+    // For serialization only. Do not use in other cases.
+    std::string var_name() const;
+
+    // For serialization only. Do not use in other cases.
+    bool is_rvar() const;
+
+    // For serialization only. Do not use in other cases.
+    bool locked() const;
+
     // Return a string of the form func.var -- note that this is safe
     // to call for root or inline LoopLevels, but asserts if !defined().
     std::string to_string() const;
@@ -424,6 +468,9 @@ struct Dim {
      * loop (see the DimType enum above). */
     DimType dim_type;
 
+    /** The strategy for loop partitioning. */
+    Partition partition_policy;
+
     /** Can this loop be evaluated in any order (including in
      * parallel)? Equivalently, are there no data hazards between
      * evaluations of the Func at distinct values of this var? */
@@ -627,8 +674,10 @@ public:
     // @{
     const LoopLevel &store_level() const;
     const LoopLevel &compute_level() const;
+    const LoopLevel &hoist_storage_level() const;
     LoopLevel &store_level();
     LoopLevel &compute_level();
+    LoopLevel &hoist_storage_level();
     // @}
 
     /** Pass an IRVisitor through to all Exprs referenced in the
@@ -652,6 +701,10 @@ public:
     }
     StageSchedule(const StageSchedule &other) = default;
     StageSchedule();
+    StageSchedule(const std::vector<ReductionVariable> &rvars, const std::vector<Split> &splits,
+                  const std::vector<Dim> &dims, const std::vector<PrefetchDirective> &prefetches,
+                  const FuseLoopLevel &fuse_level, const std::vector<FusedPair> &fused_pairs,
+                  bool touched, bool allow_race_conditions, bool atomic, bool override_atomic_associativity_test);
 
     /** Return a copy of this StageSchedule. */
     StageSchedule get_copy() const;
