@@ -9,21 +9,12 @@ Expr Simplify::visit(const Add *op, ExprInfo *bounds) {
     Expr b = mutate(op->b, &b_bounds);
 
     if (bounds && no_overflow_int(op->type)) {
-        bounds->min_defined = a_bounds.min_defined && b_bounds.min_defined;
-        bounds->max_defined = a_bounds.max_defined && b_bounds.max_defined;
-        if (add_would_overflow(64, a_bounds.min, b_bounds.min)) {
-            bounds->min_defined = false;
-            bounds->min = 0;
-        } else {
-            bounds->min = a_bounds.min + b_bounds.min;
-        }
-        if (add_would_overflow(64, a_bounds.max, b_bounds.max)) {
-            bounds->max_defined = false;
-            bounds->max = 0;
-        } else {
-            bounds->max = a_bounds.max + b_bounds.max;
-        }
-
+        bounds->min_defined = a_bounds.min_defined &&
+                              b_bounds.min_defined &&
+                              add_with_overflow(64, a_bounds.min, b_bounds.min, &(bounds->min));
+        bounds->max_defined = a_bounds.max_defined &&
+                              b_bounds.max_defined &&
+                              add_with_overflow(64, a_bounds.max, b_bounds.max, &(bounds->max));
         bounds->alignment = a_bounds.alignment + b_bounds.alignment;
         bounds->trim_bounds_using_alignment();
     }
@@ -121,6 +112,16 @@ Expr Simplify::visit(const Add *op, ExprInfo *bounds) {
              rewrite(y*x + y*z, y*(x + z)) ||
              rewrite(x*c0 + y*c1, (x + y*fold(c1/c0)) * c0, c1 % c0 == 0) ||
              rewrite(x*c0 + y*c1, (x*fold(c0/c1) + y) * c1, c0 % c1 == 0) ||
+
+             // Hoist shuffles. The Shuffle visitor wants to sink
+             // extract_elements to the leaves, and those count as degenerate
+             // slices, so only hoist shuffles that grab more than one lane.
+             rewrite(slice(x, c0, c1, c2) + slice(y, c0, c1, c2), slice(x + y, c0, c1, c2), c2 > 1 && lanes_of(x) == lanes_of(y)) ||
+             rewrite(slice(x, c0, c1, c2) + (z + slice(y, c0, c1, c2)), slice(x + y, c0, c1, c2) + z, c2 > 1 && lanes_of(x) == lanes_of(y)) ||
+             rewrite(slice(x, c0, c1, c2) + (slice(y, c0, c1, c2) + z), slice(x + y, c0, c1, c2) + z, c2 > 1 && lanes_of(x) == lanes_of(y)) ||
+             rewrite(slice(x, c0, c1, c2) + (z - slice(y, c0, c1, c2)), slice(x - y, c0, c1, c2) + z, c2 > 1 && lanes_of(x) == lanes_of(y)) ||
+             rewrite(slice(x, c0, c1, c2) + (slice(y, c0, c1, c2) - z), slice(x + y, c0, c1, c2) - z, c2 > 1 && lanes_of(x) == lanes_of(y)) ||
+
              (no_overflow(op->type) &&
               (rewrite(x + x*y, x * (y + 1)) ||
                rewrite(x + y*x, (y + 1) * x) ||
@@ -196,18 +197,6 @@ Expr Simplify::visit(const Add *op, ExprInfo *bounds) {
             return mutate(rewrite.result, bounds);
         }
         // clang-format on
-
-        const Shuffle *shuffle_a = a.as<Shuffle>();
-        const Shuffle *shuffle_b = b.as<Shuffle>();
-        if (shuffle_a && shuffle_b &&
-            shuffle_a->is_slice() &&
-            shuffle_b->is_slice()) {
-            if (a.same_as(op->a) && b.same_as(op->b)) {
-                return hoist_slice_vector<Add>(op);
-            } else {
-                return hoist_slice_vector<Add>(Add::make(a, b));
-            }
-        }
     }
 
     if (a.same_as(op->a) && b.same_as(op->b)) {
