@@ -63,7 +63,11 @@ Stmt Simplify::visit(const IfThenElse *op) {
     if (else_unreachable) {
         return then_case;
     } else if (then_unreachable) {
-        return else_case;
+        if (else_case.defined()) {
+            return else_case;
+        } else {
+            return Evaluate::make(0);
+        }
     }
 
     if (is_no_op(else_case)) {
@@ -262,13 +266,13 @@ Stmt Simplify::visit(const For *op) {
         }
         return mutate(s);
     } else if (!stmt_uses_var(new_body, op->name) && !is_const_zero(op->min)) {
-        return For::make(op->name, make_zero(Int(32)), new_extent, op->for_type, op->device_api, new_body);
+        return For::make(op->name, make_zero(Int(32)), new_extent, op->for_type, op->partition_policy, op->device_api, new_body);
     } else if (op->min.same_as(new_min) &&
                op->extent.same_as(new_extent) &&
                op->body.same_as(new_body)) {
         return op;
     } else {
-        return For::make(op->name, new_min, new_extent, op->for_type, op->device_api, new_body);
+        return For::make(op->name, new_min, new_extent, op->for_type, op->partition_policy, op->device_api, new_body);
     }
 }
 
@@ -296,12 +300,13 @@ Stmt Simplify::visit(const Store *op) {
     ExprInfo index_info;
     Expr index = mutate(op->index, &index_info);
 
-    // If the store is fully out of bounds, drop it.
+    // If the store is fully unconditional and out of bounds, drop it.
     // This should only occur inside branches that make the store unreachable,
     // but perhaps the branch was hard to prove constant true or false. This
     // provides an alternative mechanism to simplify these unreachable stores.
     string alloc_extent_name = op->name + ".total_extent_bytes";
-    if (bounds_and_alignment_info.contains(alloc_extent_name)) {
+    if (is_const_one(op->predicate) &&
+        bounds_and_alignment_info.contains(alloc_extent_name)) {
         if (index_info.max_defined && index_info.max < 0) {
             in_unreachable = true;
             return Evaluate::make(unreachable());
@@ -324,6 +329,10 @@ Stmt Simplify::visit(const Store *op) {
 
     const Load *load = value.as<Load>();
     const Broadcast *scalar_pred = predicate.as<Broadcast>();
+    if (scalar_pred && !scalar_pred->value.type().is_scalar()) {
+        // Nested vectorization
+        scalar_pred = nullptr;
+    }
 
     ModulusRemainder align = ModulusRemainder::intersect(op->alignment, base_info.alignment);
 
@@ -549,10 +558,10 @@ Stmt Simplify::visit(const Block *op) {
                equal(if_first->condition, if_next->condition) &&
                is_pure(if_first->condition)) {
         // Two ifs with matching conditions.
-        Stmt then_case = mutate(Block::make(if_first->then_case, if_next->then_case));
+        Stmt then_case = Block::make(if_first->then_case, if_next->then_case);
         Stmt else_case;
         if (if_first->else_case.defined() && if_next->else_case.defined()) {
-            else_case = mutate(Block::make(if_first->else_case, if_next->else_case));
+            else_case = Block::make(if_first->else_case, if_next->else_case);
         } else if (if_first->else_case.defined()) {
             // We already simplified the body of the ifs.
             else_case = if_first->else_case;
@@ -563,7 +572,9 @@ Stmt Simplify::visit(const Block *op) {
         if (if_rest.defined()) {
             result = Block::make(result, if_rest);
         }
-        return result;
+        // We must mutate the entire IfThenElse block without first mutating the
+        // branches to compute reachability accurately.
+        return mutate(result);
     } else if (if_first &&
                if_next &&
                !if_next->else_case.defined() &&
@@ -574,15 +585,18 @@ Stmt Simplify::visit(const Block *op) {
         // the first condition.  The second if can be nested
         // inside the first one, because if it's true the
         // first one must also be true.
-        Stmt then_case = mutate(Block::make(if_first->then_case, if_next));
+        Stmt then_case = Block::make(if_first->then_case, if_next);
         Stmt else_case = if_first->else_case;
         Stmt result = IfThenElse::make(if_first->condition, then_case, else_case);
         if (if_rest.defined()) {
             result = Block::make(result, if_rest);
         }
-        return result;
+        // As above, we must mutate the entire IfThenElse block without first
+        // mutating the branches to compute reachability accurately.
+        return mutate(result);
     } else if (if_first &&
                if_next &&
+               !if_next->else_case.defined() &&
                is_pure(if_first->condition) &&
                is_pure(if_next->condition) &&
                is_const_one(mutate(!(if_first->condition && if_next->condition), nullptr))) {
@@ -599,7 +613,7 @@ Stmt Simplify::visit(const Block *op) {
         if (if_rest.defined()) {
             result = Block::make(result, if_rest);
         }
-        return result;
+        return mutate(result);
     } else if (op->first.same_as(first) &&
                op->rest.same_as(rest)) {
         return op;
@@ -689,5 +703,14 @@ Stmt Simplify::visit(const Atomic *op) {
     }
 }
 
+Stmt Simplify::visit(const HoistedStorage *op) {
+    Stmt body = mutate(op->body);
+    if (body.same_as(op->body)) {
+        return op;
+    } else {
+        return HoistedStorage::make(op->name, body);
+    }
+}
+
 }  // namespace Internal
 }  // namespace Halide
