@@ -298,7 +298,8 @@ std::string type_to_c_type(Type type, bool include_space, bool c_plus_plus) {
                 if (modifier & halide_handle_cplusplus_type::Restrict) {
                     oss << " restrict";
                 }
-                if (modifier & halide_handle_cplusplus_type::Pointer) {
+                if ((modifier & halide_handle_cplusplus_type::Pointer) &&
+                    !(modifier & halide_handle_cplusplus_type::FunctionTypedef)) {
                     oss << " *";
                 }
             }
