@@ -1,6 +1,7 @@
 #include "Errors.h"
 #include "Halide.h"
 #include "HalidePlugin.h"
+#include "ParamParser.h"
 
 namespace Halide {
 namespace Internal {
@@ -8,6 +9,11 @@ namespace Autoscheduler {
 
 namespace {
 
+struct GradientAutoschedulerParams {
+    /** Maximum level of parallelism available. */
+    int parallelism = 16;
+};
+
 std::map<std::string, Box> inference_bounds(const std::vector<Function> &functions,
                                             const std::vector<Box> &output_bounds) {
     std::vector<Func> funcs;
@@ -86,7 +92,7 @@ int natural_vector_size(const Target &target, const Type &t) {
 
 template<typename FuncOrStage>
 void parallelize_vars_and_rvars_gpu(
-    const MachineParams &params,
+    const GradientAutoschedulerParams &params,
     FuncOrStage func_or_stage,
     bool is_pure_def,
     const std::vector<Var> &vars,
@@ -324,7 +330,7 @@ void parallelize_vars_and_rvars_gpu(
 
 template<typename FuncOrStage>
 void parallelize_vars_and_rvars_cpu(
-    const MachineParams &params,
+    const GradientAutoschedulerParams &params,
     FuncOrStage func_or_stage,
     int natural_vector_size,
     bool is_pure_def,
@@ -528,7 +534,7 @@ void parallelize_vars_and_rvars_cpu(
 
 template<typename FuncOrStage>
 void parallelize_vars_and_rvars(
-    const MachineParams &params,
+    const GradientAutoschedulerParams &params,
     FuncOrStage func_or_stage,
     int natural_vector_size,
     bool is_pure_def,
@@ -565,7 +571,7 @@ void parallelize_vars_and_rvars(
     }
 }
 
-void apply_schedule(const MachineParams &params,
+void apply_schedule(const GradientAutoschedulerParams &params,
                     const Target &target,
                     Func func,
                     int update_id,
@@ -606,10 +612,6 @@ void apply_schedule(const MachineParams &params,
         for (const ReductionVariable &r : reduction_vars) {
             rvars.emplace_back(r.var);
         }
-        int rdomain_size = 1;
-        for (int b : rvar_bounds) {
-            rdomain_size *= b;
-        }
         // Define the thresholds for the pure domain.
         // For CPU we want at least params.parallelism number of elements
         // to launch threads. For GPU we want to launch at least 64 GPU blocks.
@@ -821,7 +823,7 @@ void apply_schedule(const MachineParams &params,
 
 void generate_schedule(const std::vector<Function> &outputs,
                        const Target &target,
-                       const MachineParams &params,
+                       const GradientAutoschedulerParams &params,
                        AutoSchedulerResults *auto_scheduler_results) {
     // The first few steps are the same as src/AutoSchedule.cpp
     // Make an environment map which is used throughout the auto scheduling process.
@@ -923,18 +925,26 @@ void generate_schedule(const std::vector<Function> &outputs,
         }
     }
 
-    auto_scheduler_results->scheduler_name = "Li2018";
     auto_scheduler_results->schedule_source = schedule_source.str();
     debug(1) << schedule_source.str() << "\n";
 }
 
 struct Li2018 {
-    void operator()(const Pipeline &p, const Target &target, const MachineParams &params, AutoSchedulerResults *results) {
+    void operator()(const Pipeline &p, const Target &target, const AutoschedulerParams &params_in, AutoSchedulerResults *results) {
+        internal_assert(params_in.name == "Li2018");
+
         std::vector<Function> outputs;
         for (const Func &f : p.outputs()) {
             outputs.push_back(f.function());
         }
+        GradientAutoschedulerParams params;
+        {
+            ParamParser parser(params_in.extra);
+            parser.parse("parallelism", &params.parallelism);
+            parser.finish();
+        }
         generate_schedule(outputs, target, params, results);
+        results->autoscheduler_params = params_in;
     }
 };
 
