@@ -231,6 +231,21 @@ WEAK int halide_device_sync(void *user_context, struct halide_buffer_t *buf) {
     }
 }
 
+/**
+ * Wait for current GPU operations to complete. Calling this explicitly
+ * should rarely be necessary, except maybe for profiling.
+ * This variation of the synchronizing is useful when a synchronization is desirable
+ * without specifying any buffer to synchronize on.
+ */
+WEAK int halide_device_sync_global(void *user_context, const struct halide_device_interface_t *device_interface) {
+    if (device_interface == nullptr) {
+        return halide_error_code_no_device_interface;
+    }
+    // This function calls immediately the device_interface implementation to syncrhonize on
+    //  "no buffer" (i.e., nullptr buffer) to trigger a "global" device sync.
+    return device_interface->impl->device_sync(user_context, nullptr);
+}
+
 /** Allocate device memory to back a halide_buffer_t. */
 WEAK int halide_device_malloc(void *user_context, struct halide_buffer_t *buf,
                               const halide_device_interface_t *device_interface) {
