@@ -24,6 +24,9 @@ WEAK int halide_start_clock(void *user_context) {
 }
 
 WEAK int64_t halide_current_time_ns(void *user_context) {
+    // It is an error to call halide_current_time_ns() if halide_start_clock() has never been called
+    halide_debug_assert(user_context, halide_reference_clock_inited);
+
     return zx_clock_get_monotonic() - halide_reference_clock;
 }
 
