@@ -37,6 +37,9 @@ WEAK int halide_start_clock(void *user_context) {
 }
 
 WEAK int64_t halide_current_time_ns(void *user_context) {
+    // It is an error to call halide_current_time_ns() if halide_start_clock() has never been called
+    halide_debug_assert(user_context, halide_reference_clock_inited);
+
     uint64_t now = mach_absolute_time();
     return (now - halide_reference_clock) * halide_timebase_info.numer / halide_timebase_info.denom;
 }
