@@ -1,29 +1,18 @@
 #include "HalideRuntime.h"
 #include "runtime_internal.h"
 
-#include "printer.h"
-
 extern "C" {
 
 extern void *malloc(size_t);
 extern void free(void *);
 
 WEAK void *halide_default_malloc(void *user_context, size_t x) {
-    // Allocate enough space for aligning the pointer we return.
-    const size_t alignment = halide_malloc_alignment();
-    void *orig = malloc(x + alignment);
-    if (orig == nullptr) {
-        // Will result in a failed assertion and a call to halide_error
-        return nullptr;
-    }
-    // We want to store the original pointer prior to the pointer we return.
-    void *ptr = (void *)(((size_t)orig + alignment + sizeof(void *) - 1) & ~(alignment - 1));
-    ((void **)ptr)[-1] = orig;
-    return ptr;
+    const size_t alignment = ::halide_internal_malloc_alignment();
+    return ::halide_internal_aligned_alloc(alignment, x);
 }
 
 WEAK void halide_default_free(void *user_context, void *ptr) {
-    free(((void **)ptr)[-1]);
+    ::halide_internal_aligned_free(ptr);
 }
 }
 
