@@ -1,4 +1,5 @@
 #include "HalideRuntime.h"
+#include "runtime_atomics.h"
 
 extern "C" {
 
@@ -19,13 +20,18 @@ WEAK_INLINE int halide_profiler_set_current_func(halide_profiler_state *state, i
 
 // Invariant: shared xor local, and both are either 0 or 1. 0 means acquired.
 WEAK_INLINE int halide_profiler_acquire_sampling_token(int32_t *shared, int32_t *local) {
-    *local = __sync_lock_test_and_set(shared, 1);
+    using namespace Halide::Runtime::Internal::Synchronization;
+
+    *local = atomic_exchange_acquire(shared, 1);
     return 0;
 }
 
 WEAK_INLINE int halide_profiler_release_sampling_token(int32_t *shared, int32_t *local) {
+    using namespace Halide::Runtime::Internal::Synchronization;
+
     if (*local == 0) {
-        __sync_lock_release(shared);
+        int32_t value = 0;
+        atomic_store_release(shared, &value);
         *local = 1;
     }
     return 0;
@@ -37,10 +43,14 @@ WEAK_INLINE int halide_profiler_init_sampling_token(int32_t *sampling_token, int
 }
 
 WEAK_INLINE int halide_profiler_incr_active_threads(halide_profiler_state *state) {
-    return __sync_fetch_and_add(&(state->active_threads), 1);
+    using namespace Halide::Runtime::Internal::Synchronization;
+
+    return atomic_fetch_add_sequentially_consistent(&(state->active_threads), 1);
 }
 
 WEAK_INLINE int halide_profiler_decr_active_threads(halide_profiler_state *state) {
-    return __sync_fetch_and_sub(&(state->active_threads), 1);
+    using namespace Halide::Runtime::Internal::Synchronization;
+
+    return atomic_fetch_sub_sequentially_consistent(&(state->active_threads), 1);
 }
 }
