@@ -1,5 +1,6 @@
 #include "HalideRuntime.h"
 #include "printer.h"
+#include "runtime_atomics.h"
 #include "scoped_spin_lock.h"
 
 extern "C" {
@@ -14,8 +15,7 @@ namespace Internal {
 // A spinlock that allows for shared and exclusive access. It's
 // equivalent to a reader-writer lock, but in my case the "readers"
 // will actually be writing simultaneously to the trace buffer, so
-// that's a bad name. We use the __sync primitives used elsewhere in
-// the runtime for atomic work. They are well supported by clang.
+// that's a bad name.
 class SharedExclusiveSpinLock {
     volatile uint32_t lock = 0;
 
@@ -37,37 +37,51 @@ class SharedExclusiveSpinLock {
 
 public:
     ALWAYS_INLINE void acquire_shared() {
+        using namespace Halide::Runtime::Internal::Synchronization;
+
         while (true) {
-            uint32_t x = lock & shared_mask;
-            if (__sync_bool_compare_and_swap(&lock, x, x + 1)) {
+            uint32_t expected = lock & shared_mask;
+            uint32_t desired = expected + 1;
+            if (atomic_cas_strong_sequentially_consistent(&lock, &expected, &desired)) {
                 return;
             }
         }
     }
 
     ALWAYS_INLINE void release_shared() {
-        __sync_fetch_and_sub(&lock, 1);
+        using namespace Halide::Runtime::Internal::Synchronization;
+
+        atomic_fetch_sub_sequentially_consistent(&lock, (uint32_t)1);
     }
 
     ALWAYS_INLINE void acquire_exclusive() {
+        using namespace Halide::Runtime::Internal::Synchronization;
+
         while (true) {
             // If multiple threads are trying to acquire exclusive
             // ownership, we may need to rerequest exclusive waiting
             // while we spin, as it gets unset whenever a thread
             // acquires exclusive ownership.
-            __sync_fetch_and_or(&lock, exclusive_waiting_mask);
-            if (__sync_bool_compare_and_swap(&lock, exclusive_waiting_mask, exclusive_held_mask)) {
+            atomic_fetch_or_sequentially_consistent(&lock, exclusive_waiting_mask);
+            uint32_t expected = exclusive_waiting_mask;
+            uint32_t desired = exclusive_held_mask;
+            if (atomic_cas_strong_sequentially_consistent(&lock, &expected, &desired)) {
                 return;
             }
         }
     }
 
     ALWAYS_INLINE void release_exclusive() {
-        __sync_fetch_and_and(&lock, ~exclusive_held_mask);
+        using namespace Halide::Runtime::Internal::Synchronization;
+
+        atomic_fetch_and_sequentially_consistent(&lock, ~exclusive_held_mask);
     }
 
     ALWAYS_INLINE void init() {
-        lock = 0;
+        using namespace Halide::Runtime::Internal::Synchronization;
+
+        uint32_t value = 0;
+        atomic_store_sequentially_consistent(&lock, &value);
     }
 
     SharedExclusiveSpinLock() = default;
@@ -83,15 +97,17 @@ class TraceBuffer {
     // Attempt to atomically acquire space in the buffer to write a
     // packet. Returns nullptr if the buffer was full.
     ALWAYS_INLINE halide_trace_packet_t *try_acquire_packet(void *user_context, uint32_t size) {
+        using namespace Halide::Runtime::Internal::Synchronization;
+
         lock.acquire_shared();
         halide_abort_if_false(user_context, size <= buffer_size);
-        uint32_t my_cursor = __sync_fetch_and_add(&cursor, size);
+        uint32_t my_cursor = atomic_fetch_add_sequentially_consistent(&cursor, size);
         if (my_cursor + size > sizeof(buf)) {
             // Don't try to back it out: instead, just allow this request to fail
             // (along with all subsequent requests) and record the 'overage'
             // that was added and should be ignored; then, in the next flush,
             // remove the overage.
-            __sync_fetch_and_add(&overage, size);
+            atomic_fetch_add_sequentially_consistent(&overage, size);
             lock.release_shared();
             return nullptr;
         } else {
@@ -131,8 +147,10 @@ public:
 
     // Release a packet, allowing it to be written out with flush
     ALWAYS_INLINE void release_packet(halide_trace_packet_t *) {
+        using namespace Halide::Runtime::Internal::Synchronization;
+
         // Need a memory barrier to guarantee all the writes are done.
-        __sync_synchronize();
+        atomic_thread_fence_sequentially_consistent();
         lock.release_shared();
     }
 
@@ -158,9 +176,11 @@ WEAK void *halide_trace_file_internally_opened = nullptr;
 extern "C" {
 
 WEAK int32_t halide_default_trace(void *user_context, const halide_trace_event_t *e) {
+    using namespace Halide::Runtime::Internal::Synchronization;
+
     static int32_t ids = 1;
 
-    int32_t my_id = __sync_fetch_and_add(&ids, 1);
+    int32_t my_id = atomic_fetch_add_sequentially_consistent(&ids, 1);
 
     // If we're dumping to a file, use a binary format
     int fd = halide_get_trace_file(user_context);
@@ -347,7 +367,7 @@ WEAK int halide_get_trace_file(void *user_context) {
     if (halide_trace_file < 0) {
         const char *trace_file_name = getenv("HL_TRACE_FILE");
         if (trace_file_name) {
-            void *file = fopen(trace_file_name, "ab");
+            void *file = halide_fopen(trace_file_name, "ab");
             halide_abort_if_false(user_context, file && "Failed to open trace file\n");
             halide_set_trace_file(fileno(file));
             halide_trace_file_internally_opened = file;
@@ -375,10 +395,12 @@ WEAK int halide_shutdown_trace() {
         if (halide_trace_buffer) {
             free(halide_trace_buffer);
         }
-        return ret;
-    } else {
-        return 0;
+        if (ret != 0) {
+            return halide_error_code_trace_failed;
+        }
+        // else fall thru
     }
+    return halide_error_code_success;
 }
 
 namespace {
