@@ -39,25 +39,17 @@ class VulkanContext {
     void *user_context;
 
 public:
-    VulkanMemoryAllocator *allocator;
-    VkInstance instance;
-    VkDevice device;
-    VkCommandPool command_pool;
-    VkPhysicalDevice physical_device;
-    VkQueue queue;
-    uint32_t queue_family_index;  // used for operations requiring queue family
-    halide_error_code_t error;
+    VulkanMemoryAllocator *allocator = nullptr;
+    VkInstance instance = nullptr;
+    VkDevice device = nullptr;
+    VkCommandPool command_pool = 0;
+    VkPhysicalDevice physical_device = nullptr;
+    VkQueue queue = nullptr;
+    uint32_t queue_family_index = 0;  // used for operations requiring queue family
+    halide_error_code_t error = halide_error_code_success;
 
-    HALIDE_ALWAYS_INLINE VulkanContext(void *user_context)
-        : user_context(user_context),
-          allocator(nullptr),
-          instance(nullptr),
-          device(nullptr),
-          command_pool(0),
-          physical_device(nullptr),
-          queue(nullptr),
-          queue_family_index(0),
-          error(halide_error_code_success) {
+    HALIDE_ALWAYS_INLINE explicit VulkanContext(void *user_context)
+        : user_context(user_context) {
 
         int result = halide_vulkan_acquire_context(user_context,
                                                    reinterpret_cast<halide_vulkan_memory_allocator **>(&allocator),
@@ -320,13 +312,13 @@ int vk_create_device(void *user_context, const StringTable &requested_layers, Vk
     }
 
     debug(user_context) << "  found " << (uint32_t)required_device_extensions.size() << " required extensions for device!\n";
-    for (int n = 0; n < (int)required_device_extensions.size(); ++n) {
+    for (uint32_t n = 0; n < required_device_extensions.size(); ++n) {
         debug(user_context) << "   required extension: " << required_device_extensions[n] << "\n";
     }
 
     // enable all available optional extensions
     debug(user_context) << "  checking for " << (uint32_t)optional_device_extensions.size() << " optional extensions for device ...\n";
-    for (int n = 0; n < (int)optional_device_extensions.size(); ++n) {
+    for (uint32_t n = 0; n < optional_device_extensions.size(); ++n) {
         if (supported_device_extensions.contains(optional_device_extensions[n])) {
             debug(user_context) << "   optional extension: " << optional_device_extensions[n] << "\n";
             required_device_extensions.append(user_context, optional_device_extensions[n]);
