@@ -8,7 +8,7 @@ using namespace Halide;
 bool custom_malloc_called = false;
 bool custom_free_called = false;
 
-void *my_malloc(void *user_context, size_t x) {
+void *my_malloc(JITUserContext *user_context, size_t x) {
     custom_malloc_called = true;
     void *orig = malloc(x + 32);
     void *ptr = (void *)((((size_t)orig + 32) >> 5) << 5);
@@ -16,14 +16,14 @@ void *my_malloc(void *user_context, size_t x) {
     return ptr;
 }
 
-void my_free(void *user_context, void *ptr) {
+void my_free(JITUserContext *user_context, void *ptr) {
     custom_free_called = true;
     free(((void **)ptr)[-1]);
 }
 
 int main(int argc, char **argv) {
     if (get_jit_target_from_environment().arch == Target::WebAssembly) {
-        printf("[SKIP] WebAssembly JIT does not support set_custom_allocator().\n");
+        printf("[SKIP] WebAssembly JIT does not support custom allocators.\n");
         return 0;
     }
 
@@ -34,7 +34,8 @@ int main(int argc, char **argv) {
     g(x) = f(x);
     f.compute_root();
 
-    g.set_custom_allocator(my_malloc, my_free);
+    g.jit_handlers().custom_malloc = my_malloc;
+    g.jit_handlers().custom_free = my_free;
 
     Buffer<int> im = g.realize({100000});
 
