@@ -46,6 +46,6 @@ int main(int argc, char **argv) {
         printf("Success!\n");
         return 0;
     } else {
-        return -1;
+        return 1;
     }
 }
