@@ -324,9 +324,11 @@ int main(int argc, char **argv) {
 
     // Only for 64bit target with fp16 feature
     if (!(hl_target.arch == Target::ARM && hl_target.bits == 64 && hl_target.has_feature(Target::ARMFp16))) {
+        Halide::Internal::Test::Sharder::accept_sharded_status();
         printf("[SKIP] To run this test, set HL_TARGET=arm-64-<os>-arm_fp16. \n");
         return 0;
     }
+
     // Create Test Object
     // Use smaller dimension than default(768, 128) to avoid fp16 overflow in reduction test case
     SimdOpCheck test(hl_target, 384, 32);
@@ -337,27 +339,12 @@ int main(int argc, char **argv) {
 
     if (argc > 1) {
         test.filter = argv[1];
-        test.set_num_threads(1);
     }
 
     if (getenv("HL_SIMD_OP_CHECK_FILTER")) {
         test.filter = getenv("HL_SIMD_OP_CHECK_FILTER");
     }
 
-    // TODO: multithreading here is the cause of https://github.com/halide/Halide/issues/3669;
-    // the fundamental issue is that we make one set of ImageParams to construct many
-    // Exprs, then realize those Exprs on arbitrary threads; it is known that sharing
-    // one Func across multiple threads is not guaranteed to be safe, and indeed, TSAN
-    // reports data races, of which some are likely 'benign' (e.g. Function.freeze) but others
-    // are highly suspect (e.g. Function.lock_loop_levels). Since multithreading here
-    // was added just to avoid having this test be the last to finish, the expedient 'fix'
-    // for now is to remove the multithreading. A proper fix could be made by restructuring this
-    // test so that every Expr constructed for testing was guaranteed to share no Funcs
-    // (Function.deep_copy() perhaps). Of course, it would also be desirable to allow Funcs, Exprs, etc
-    // to be usable across multiple threads, but that is a major undertaking that is
-    // definitely not worthwhile for present Halide usage patterns.
-    test.set_num_threads(1);
-
     if (argc > 2) {
         // Don't forget: if you want to run the standard tests to a specific output
         // directory, you'll need to invoke with the first arg enclosed
@@ -371,7 +358,7 @@ int main(int argc, char **argv) {
     bool success = test.test_all();
 
     if (!success) {
-        return -1;
+        return 1;
     }
 
     printf("Success!\n");
