@@ -28,7 +28,7 @@ int main(int argc, char **argv) {
         fprintf(stderr, "Err: f(50, 100) = %d (supposed to be 123)\n"
                         "f(99, 199) = %d (supposed to be 234)\n",
                 result(50, 100), result(99, 199));
-        return -1;
+        return 1;
     }
 
     printf("Success!\n");
