@@ -15,7 +15,7 @@ int main(int argc, char **argv) {
     buf.data()[0] = 17;
     if (buf(100, 300, 500, 400) != 17) {
         printf("Image indexing into buffers with non-zero mins is broken\n");
-        return -1;
+        return 1;
     }
 
     printf("Success!\n");
