@@ -3,8 +3,10 @@
 
 #include "Halide.h"
 #include "halide_test_dirs.h"
+#include "test_sharding.h"
 
 #include <fstream>
+#include <iostream>
 
 namespace Halide {
 struct TestResult {
@@ -21,6 +23,13 @@ struct Task {
 
 class SimdOpCheckTest {
 public:
+    static constexpr int max_i8 = 127;
+    static constexpr int max_i16 = 32767;
+    static constexpr int max_i32 = 0x7fffffff;
+    static constexpr int max_u8 = 255;
+    static constexpr int max_u16 = 65535;
+    const Expr max_u32 = UInt(32).max();
+
     std::string filter{"*"};
     std::string output_directory{Internal::get_test_tmp_dir()};
     std::vector<Task> tasks;
@@ -46,14 +55,14 @@ public:
     int W;
     int H;
 
+    using Sharder = Halide::Internal::Test::Sharder;
+
     SimdOpCheckTest(const Target t, int w, int h)
         : target(t), W(w), H(h) {
         target = target
                      .with_feature(Target::NoBoundsQuery)
                      .with_feature(Target::NoAsserts)
-                     .with_feature(Target::NoRuntime)
-                     .with_feature(Target::DisableLLVMLoopOpt);
-        num_threads = Internal::ThreadPool<void>::num_processors_online();
+                     .with_feature(Target::NoRuntime);
     }
     virtual ~SimdOpCheckTest() = default;
 
@@ -61,19 +70,9 @@ public:
         rng.seed(seed);
     }
 
-    size_t get_num_threads() const {
-        return num_threads;
-    }
-
-    void set_num_threads(size_t n) {
-        num_threads = n;
-    }
-
     virtual bool can_run_code() const {
-        // Assume we are configured to run wasm if requested
-        // (we'll fail further downstream if not)
         if (target.arch == Target::WebAssembly) {
-            return true;
+            return Halide::Internal::WasmModule::can_jit_target(Target("wasm-32-wasmrt"));
         }
         // If we can (target matches host), run the error checking Halide::Func.
         Target host_target = get_host_target();
@@ -83,12 +82,29 @@ public:
              target.os == host_target.os);
         // A bunch of feature flags also need to match between the
         // compiled code and the host in order to run the code.
-        for (Target::Feature f : {Target::SSE41, Target::AVX,
-                                  Target::AVX2, Target::AVX512,
-                                  Target::FMA, Target::FMA4, Target::F16C,
-                                  Target::VSX, Target::POWER_ARCH_2_07,
-                                  Target::ARMv7s, Target::NoNEON,
-                                  Target::WasmSimd128}) {
+        for (Target::Feature f : {
+                 Target::ARMDotProd,
+                 Target::ARMFp16,
+                 Target::ARMv7s,
+                 Target::ARMv81a,
+                 Target::AVX,
+                 Target::AVX2,
+                 Target::AVX512,
+                 Target::AVX512_Cannonlake,
+                 Target::AVX512_KNL,
+                 Target::AVX512_SapphireRapids,
+                 Target::AVX512_Skylake,
+                 Target::F16C,
+                 Target::FMA,
+                 Target::FMA4,
+                 Target::NoNEON,
+                 Target::POWER_ARCH_2_07,
+                 Target::RVV,
+                 Target::SSE41,
+                 Target::SVE,
+                 Target::SVE2,
+                 Target::VSX,
+             }) {
             if (target.has_feature(f) != host_target.has_feature(f)) {
                 can_run_the_code = false;
             }
@@ -114,7 +130,7 @@ public:
         bool found_it = false;
 
         std::ostringstream msg;
-        msg << op << " did not generate for target=" << target.to_string() << " vector_width=" << vector_width << ". Instead we got:\n";
+        msg << op << " did not generate for target=" << get_run_target().to_string() << " vector_width=" << vector_width << ". Instead we got:\n";
 
         std::string line;
         while (getline(asm_file, line)) {
@@ -171,6 +187,13 @@ public:
         return wildcard_match("*" + p + "*", str);
     }
 
+    Target get_run_target() const {
+        return target
+            .without_feature(Target::NoRuntime)
+            .without_feature(Target::NoAsserts)
+            .without_feature(Target::NoBoundsQuery);
+    }
+
     TestResult check_one(const std::string &op, const std::string &name, int vector_width, Expr e) {
         std::ostringstream error_msg;
 
@@ -231,10 +254,7 @@ public:
 
         bool can_run_the_code = can_run_code();
         if (can_run_the_code) {
-            Target run_target = target
-                                    .without_feature(Target::NoRuntime)
-                                    .without_feature(Target::NoAsserts)
-                                    .without_feature(Target::NoBoundsQuery);
+            Target run_target = get_run_target();
 
             error.infer_input_bounds({}, run_target);
             // Fill the inputs with noise
@@ -321,18 +341,20 @@ public:
     virtual bool test_all() {
         /* First add some tests based on the target */
         add_tests();
-        Internal::ThreadPool<TestResult> pool(num_threads);
-        std::vector<std::future<TestResult>> futures;
-        for (const Task &task : tasks) {
-            futures.push_back(pool.async([this, task]() {
-                return check_one(task.op, task.name, task.vector_width, task.expr);
-            }));
-        }
 
+        // Remove irrelevant noise from output
+        const Target run_target = get_run_target();
+        const std::string run_target_str = run_target.to_string();
+
+        Sharder sharder;
         bool success = true;
-        for (auto &f : futures) {
-            const TestResult &result = f.get();
-            std::cout << result.op << "\n";
+        for (size_t t = 0; t < tasks.size(); t++) {
+            if (!sharder.should_run(t)) continue;
+            const auto &task = tasks.at(t);
+            auto result = check_one(task.op, task.name, task.vector_width, task.expr);
+            constexpr int tabstop = 32;
+            const int spaces = std::max(1, tabstop - (int)result.op.size());
+            std::cout << result.op << std::string(spaces, ' ') << "(" << run_target_str << ")\n";
             if (!result.error_msg.empty()) {
                 std::cerr << result.error_msg;
                 success = false;
@@ -342,9 +364,64 @@ public:
         return success;
     }
 
+    template<typename SIMDOpCheckT>
+    static int main(int argc, char **argv, const std::vector<Target> &targets_to_test) {
+        Target host = get_host_target();
+        std::cout << "host is:      " << host << "\n";
+
+        const int seed = argc > 2 ? atoi(argv[2]) : time(nullptr);
+        std::cout << "simd_op_check test seed: " << seed << "\n";
+
+        for (const auto &t : targets_to_test) {
+            if (!t.supported()) {
+                std::cout << "[SKIP] Unsupported target: " << t << "\n";
+                return 0;
+            }
+            SIMDOpCheckT test(t);
+
+            if (!t.supported()) {
+                std::cout << "Halide was compiled without support for " << t.to_string() << ". Skipping.\n";
+                continue;
+            }
+
+            if (argc > 1) {
+                test.filter = argv[1];
+            }
+
+            if (getenv("HL_SIMD_OP_CHECK_FILTER")) {
+                test.filter = getenv("HL_SIMD_OP_CHECK_FILTER");
+            }
+
+            test.set_seed(seed);
+
+            if (argc > 2) {
+                // Don't forget: if you want to run the standard tests to a specific output
+                // directory, you'll need to invoke with the first arg enclosed
+                // in quotes (to avoid it being wildcard-expanded by the shell):
+                //
+                //    correctness_simd_op_check "*" /path/to/output
+                //
+                test.output_directory = argv[2];
+            }
+
+            bool success = test.test_all();
+
+            // Compile a runtime for this target, for use in the static test.
+            compile_standalone_runtime(test.output_directory + "simd_op_check_runtime.o", test.target);
+
+            if (!success) {
+                return 1;
+            }
+        }
+
+        std::cout << "Success!\n";
+        return 0;
+    }
+
 private:
-    size_t num_threads;
     const Halide::Var x{"x"}, y{"y"};
 };
+
 }  // namespace Halide
+
 #endif  // SIMD_OP_CHECK_H
