@@ -4,6 +4,11 @@
 using namespace Halide;
 
 int main(int argc, char **argv) {
+#ifdef WITH_SERIALIZATION_JIT_ROUNDTRIP_TESTING
+    printf("[SKIP] Serialization won't preserve GPU buffers, skipping.\n");
+    return 0;
+#endif
+
     if (!get_jit_target_from_environment().has_gpu_feature()) {
         printf("[SKIP] No GPU target enabled.\n");
         return 0;
